
#include <stdio.h>
#include <string.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <semaphore.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <time.h>
#include <linux/fd.h>
#include <signal.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>




#include <asterisk.h>
#include <asterisk/logger.h>

#include "gsmctl.h"


#include "CmdLine.h"
#include "Monitor.h"
#include "MonCmds.h"
#include "PlxApi.h"
#include "PciRegs.h"
#include "RegDefs.h"
#include "ConsFunc.h"
#include "PlxInit.h"
#include "PlxApi.h"
#include "PexApi.h"

/* This is the global thread variable for the PLX Interupt Thread..... -- ALLO_GSM -- 24-11-2011 */
static pthread_t pth_plx_thread;
static int plx_shutdown  = 0; /* This is the flag for the plx interupt thread to come out of infinite while loop --- ALLO_GSM -- 24-11-2011 */
static int plx_initialised=0;
pthread_mutex_t *init_mod_lock_ptr=NULL;
pthread_mutex_t init_mod_lock;


/* when DEFAULT_GSM_SEND_DELAY is defined, all delays after sending a messsage will
 * be at this value in milliseconds. */

static int debug = 0; /* SAM DBG */
static int at_debug[17] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; 
FILE *debugfp = NULL;


void (*cbEvents)(int port, char *event);

void gsm_close_port(int port);

static int _gsm_send_command(int port, char *s, int delay);

void gsm_set_debugfile(char *debugfile)
{

}

void gsm_set_debug_port(int port, int debuglevel)
{
	at_debug[port] = debuglevel;
//	plx_debug(debug);
}

void gsm_set_debuglevel(int debuglevel)
{
	debug = debuglevel;
	plx_debug(debug);
}

/**Port List helper*/
struct ctlport *getctlport(int port)
{
	int i;

	if (!port) return NULL;

	for (i=1; i<MAX_GSM_PORTS; i++) {
		if (ctlport_array[i].port == port) return &ctlport_array[i];
	}
	return NULL;
}

struct ctlport *getctlport_bypriv(void * priv)
{
	int i;

	if (!priv) return NULL;

	for (i=1; i<MAX_GSM_PORTS; i++) {
		if (ctlport_array[i].priv == priv) return &ctlport_array[i];
	}
	return NULL;
}

/* IF Functions */
int gsm_port_is_valid(int port)
{
	struct ctlport *ctl=getctlport(port);

	if (ctl){
		//ast_log(LOG_ERROR , "[ALLO_GSM] WE are returning 1 as we found the ctl\n" );
		return 1;
	}else{
		//ast_log(LOG_ERROR , "[ALLO_GSM] Ops!!!!! we did not found the ctl......................\n" );
		return 0;
	}
}

int gsm_port_fetch(int port)
{
	struct ctlport *ctl=getctlport(port);
	if (ctl) {
		if (!ctl->in_use) {
			ctl->in_use=1;
			return 1;
		}
	}
	return 0;
}

int gsm_port_release(int port)
{

	struct ctlport *ctl=getctlport(port);
	if (ctl) {
		ctl->in_use=0;
	}
	return 0;
}

int gsm_port_in_use_get(int port)
{
	struct ctlport *ctl=getctlport(port);
	if (ctl) {
		return ctl->in_use;
	}
	return 0;

}

int gsm_sms_enter(int port)
{
	struct ctlport *ctl=getctlport(port);

	ctl->state=CTLPORT_STATE_SMS;
	return 0;
}

int gsm_simslot_get(int port)
{
	char buf[16];
	char path[128];
	FILE *fp;

#if 0
	sprintf(path,"/sys/class/gsm/ctl%d/channel_sim_slot",port);
	fp=fopen(path,"r");
	if (fp) {
		fgets(buf, sizeof(buf), fp);
		fclose(fp);
		return strtol(buf, NULL, 10);
	} else {
		printf("Could not open sysfs interface for port %d\n", port);
	}
#else
	return port ;
#endif

	return 0;
}

int gsm_simslot_next_get(int port)
{
	char buf[16];
	char path[128];
	FILE *fp;

#if 0
	sprintf(path,"/sys/class/gsm/ctl%d/channel_sim_slot_next",port);
	fp=fopen(path,"r");
	if (fp) {
		fgets(buf, sizeof(buf), fp);
		fclose(fp);
		return strtol(buf, NULL, 10);
	} else {
		printf("Could not open sysfs interface for port %d\n", port);
		return -1;
	}
#else
	return port+1 ;
#endif

	return 0;
}

int gsm_set_next_simslot(int port, int slot)
{
	char path[128];
	FILE *fp;

#if 0
	if (gsm_simslot_get(port)==slot)
		return 0;

	sprintf(path,"/sys/class/gsm/ctl%d/channel_sim_slot_next",port);
	fp=fopen(path,"w");
	if (fp) {
		fprintf(fp,"%d", slot?1:0);
		fclose(fp);
		return 1;
	} else {
		printf("Could not open sysfs interface for port %d\n", port);
		return -1;
	}
#endif

	return 0;

}

int gsm_check_port(int port)
{
	char path[128];
	struct stat mstat;

	sprintf(path,"/dev/gsm/ctl%d",port);

	if (stat(path, &mstat) == 0) {
		return 1;
	} else {
		return 0;
	}
}

int gsm_set_led(int port,int ledstate)
{
	struct ctlport *ctl=getctlport(port);
	int fd;

      	if (!ctl || !ctl->fpw) return 0;

	fd=fileno(ctl->fpw);

	if (fd<=0) return 0;

	return ioctl( fd, GSM_SETLED, ledstate);
}

int gsm_audio_start(int port)
{
	/* We don't need to do anything here so commenting.... --- ALLO_GSM -- 18-10-2011 */
#if 1
	struct ctlport *ctl=getctlport(port);
	if (ctl)
		ctl->write_pkt_len = 160;/* default we read the 20ms packet from the PLX -- ALLO_GSM -- 21-12-2011 */
      	//return ioctl( ctl->fd, GSM_SETSTATE, STATE_CONNECTED);
#endif
	return 1;
}

int gsm_audio_stop(int port)
{
	struct ctlport *ctl=getctlport(port);
	if (ctl){
		ctl->thread_exit = 1;
		ctl->write_pkt_len = 160;/* default we read the 20ms packet from the PLX -- ALLO_GSM -- 21-12-2011 */
	}
#if 0
	/* Here delete the FIFO and recreate the same FIFO ....  -- ALLO_GSM -- 08-12-2011 */
	close(ctl->audio_fd);
	close(ctl->dummy_audio_fd);

	unlink(ctl->soc_filename);
	if (access(ctl->soc_filename, F_OK) == -1) {
	        int res = mkfifo(ctl->soc_filename, 0777);
        	if (res != 0) {
	            fprintf(stderr, "Could not create AUDIO FIFO %s\n" , ctl->soc_filename );
	            //exit(1);
        	}
    	}
	ctl->audio_fd=open(ctl->soc_filename, O_RDWR );
	if(ctl->audio_fd < 0){
		perror("AUDIO FIFO open failed ???? ");
	}
	ctl->dummy_audio_fd = open(ctl->soc_filename , O_WRONLY | O_NDELAY );
	fprintf(stderr,"[ALLO_GSM] are we stuck here (5) .audio_fd '%d' .dummy_audio_fd '%d' ??????????????????????????\n" , ctl->audio_fd , ctl->dummy_audio_fd );

	fcntl(ctl->audio_fd, F_SETFL, O_NONBLOCK);
#endif
	return 1;
}

/* This is the second type of the polling used with the Interupt from the GSM board...... */
void gsm_chan_read_flag( int port )
{

	struct ctlport *ctl= getctlport(port);
	int s, t, len;
	struct sockaddr_un remote;
	char str[32]="test";

	if(!ctl){
		ast_log(LOG_ERROR , "[ALLO_GSM] This is an invalid port '%d' !!!!!!! we did not got the ctlport \n" , port );
		return;
	}
	
	ctl->thread_exit = 0;
	if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
		perror(" ( gsm_chan_read_flag ) socket");
		return;
	}
	
	remote.sun_family = AF_UNIX;
	strcpy(remote.sun_path, ctl->soc_filename );
	len = strlen(remote.sun_path) + sizeof(remote.sun_family);
	if (connect(s, (struct sockaddr *)&remote, len) == -1) {
		perror(" ( gsm_chan_read_flag ) connect");
		close(s);
		return;
	}
	
	//printf("Connected.\n");

	//while(!ctl->thread_exit){
		if (send(s, str, strlen(str), 0) == -1) {
			perror(" ( gsm_chan_read_flag ) send");
			//return;
		}
	//	usleep(19000); /* Original figure....*/
		//usleep(30000);
	//}
	close(s);
	return;

}

/* This is the first method to control the audio read from the GSM board...... */
void * poll_control_thread ( void *b)
{

	struct ctlport *ctl= b;
	int s, t, len;
	struct sockaddr_un remote;
	char str[32]="test";
	
	ctl->thread_exit = 0;
	if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
		perror(" ( poll_control_thread ) socket");
		return;
	}
	
	//printf("Trying to connect...\n");
	
	remote.sun_family = AF_UNIX;
	strcpy(remote.sun_path, ctl->soc_filename );
	len = strlen(remote.sun_path) + sizeof(remote.sun_family);
	if (connect(s, (struct sockaddr *)&remote, len) == -1) {
		perror(" ( poll_control_thread ) connect");
		return;
	}
	
	//printf("Connected.\n");

	while(!ctl->thread_exit){
		if (send(s, str, strlen(str), 0) == -1) {
			perror(" ( poll_control_thread ) send");
			//return;
		}
		usleep(19000); /* Original figure....*/
		//usleep(30000);
	}
	return;
}

int gsm_audio_open(int port)
{
	struct ctlport *ctl=getctlport(port);

	fprintf(stderr,"[ALLO_GSM] we are returning the audio_d '%d' \n" , ctl->audio_fd );
	//return dup(ctl->audio_fd);
	return ctl->audio_fd;
}

int gsm_get_channel_mask(void)
{
#if 0
	const char taco_proc_entry[] = "/proc/driver/taco";
	const char taco_fmt[] = "%*s %*s %[^/]%*1c%s";
	const int taco_param_count = 2;
	char mod1[GSM_MAX_STR];
	char mod2[GSM_MAX_STR];
	FILE *taco_fp;
	int result = 0;

	/* open the procfs entry */
	if  (!(taco_fp = fopen(taco_proc_entry, "r"))) {
		ast_log(LOG_ERROR, "could not open %s\n", taco_proc_entry);
		result = -1;
		return (result);
	}

	/* parse the channel configuration */
	if (fscanf(taco_fp, taco_fmt, mod1, mod2) != taco_param_count) {
		fclose(taco_fp);
		result = -2;
		return (result);
	}


	/* close the file */
	fclose(taco_fp);

	/* print it to check */
	if (debug)
		ast_log(LOG_DEBUG, "mod1=(%s) mod2=(%s)\n", mod1, mod2);

	/* we will need to extend this to single channel GSM modules */
	if (!strncmp(mod1, "GSM", strlen("GSM")))
		result |= 0x03;
	 
	if (!strncmp(mod2, "GSM", strlen("GSM")))
		result |= 0x30;
	return (result); 
#endif
	return 0x00 ;
}

int gsm_get_channel_offset(void)
{
	static int channel_mask = -1;
	int channel_offset = 0;

	if (channel_mask < 0) {
		channel_mask = gsm_get_channel_mask();
	}

	switch (channel_mask) {
		case 0x03:	/* module A only */
			channel_offset = 0;
			break;
		case 0x30:	/* module B only */
			channel_offset = 4;
			break;
		case 0x33:	/* modules A and B */
			channel_offset = 0;
			break;
		case 0x00:	/* no GSM modules */
			break;
		default:
			break;
	}

	return (channel_offset);
}

int gsm_translate_channel(int *result, int channel_id)
{
	const int lookup_table[] = { 1, 2, 5, 6 };

	int channel_offset = gsm_get_channel_offset();

	if ((channel_id >= 1) && (channel_id <= 4)) { 
		*result = lookup_table[channel_id - 1] + channel_offset;
		return 0;
	} else {
		return -1;
	}
}

void gsm_audio_close(int port)
{
	struct ctlport *ctl=getctlport(port);
	close(ctl->audio_fd);
	close(ctl->dummy_audio_fd);
	close(ctl->fpr_cmd);
	close(ctl->dummy_fifo_fd);
}

void *gsm_get_priv(int port)
{
	struct ctlport *ctl=getctlport(port);
	if(!ctl)
		return NULL;
	return ctl->priv;
}

void gsm_put_priv(int port, void *priv)
{
	struct ctlport *ctl=getctlport(port);
	ctl->priv=priv;
}

void gsm_post_next_msg(int port)
{
	struct ctlport *ctl=getctlport(port);
	int m_count;

	if (!gsm_port_is_valid(port)) {
		ast_log(LOG_DEBUG, "port=(%d): %s() invalid port number\n",
			port, __FUNCTION__);
		return;
	}

	if (sem_getvalue(&avail_sem[ctl->port], &m_count)==0) {
		if (debug) { 
			struct timeval tv;
        		gettimeofday(&tv, NULL);
			ast_log(LOG_DEBUG, "port=(%d): POSTING avail_sem(%d) secs(%ld) usecs(%ld)\n", ctl->port, m_count, (long)tv.tv_sec, (long)tv.tv_usec);
		}
		if (m_count == 0) {
			gsm_stop_check_timer(ctl->port);
			sem_post(&avail_sem[ctl->port]);
		}
	} else {
		struct timeval tv;
        	gettimeofday(&tv, NULL);
		ast_log(LOG_ERROR, "port=(%d): gsm_post_next_msg() secs(%ld) usecs(%ld) could not read semaphore\n", ctl->port, (long)tv.tv_sec, (long)tv.tv_usec);
	}
}

void gsm_check_timer_expired(sigval_t arg)
{
	int port = arg.sival_int;
	struct timeval tv;

	if (!gsm_port_is_valid(port)) {
		ast_log(LOG_ERROR, "port=(%d): %s() invalid port number\n",
			port, __FUNCTION__);
		return;
	}

	gettimeofday(&tv, NULL);

	ast_log(LOG_ERROR, "port=(%d): chan_gsm port() TIMED OUT secs(%ld) usecs(%ld) last command (%s)\n", 
		port, (long)tv.tv_sec, (long)tv.tv_usec, gsm_get_last_command(port));

	gsm_post_next_msg(port);
}

void gsm_start_check_timer(int port)
{
	struct itimerspec ts;
	struct sigevent se;
	int status;

	if (!gsm_port_is_valid(port)) {
		ast_log(LOG_ERROR, "port=(%d): %s() invalid port number\n",
			port, __FUNCTION__);
		return;
	}

	if (wdog_timer[port] == INVALID_TIMER_ID) {
		se.sigev_notify = SIGEV_THREAD;
		se.sigev_value.sival_int = port;
		se.sigev_notify_function = gsm_check_timer_expired;
		se.sigev_notify_attributes = NULL;

		if ((status = timer_create(CLOCK_REALTIME, &se, &wdog_timer[port])) < 0) {
			ast_log(LOG_ERROR, "port=(%d) %s() failed timer_create()\n",
				port, __FUNCTION__);
			return;
		}

		if (debug)
			ast_log(LOG_ERROR, "port=(%d) %s() created timer_id(%d)\n",
				port, __FUNCTION__, (int)wdog_timer[port]);
	}

	ts.it_value.tv_sec = GSM_WAIT_TIMEOUT;
	ts.it_value.tv_nsec = 0;
	ts.it_interval.tv_sec = 0;
	ts.it_interval.tv_nsec = 0;

	if ((status = timer_settime(wdog_timer[port], 0, &ts, 0)) < 0) {
		ast_log(LOG_ERROR, "port=(%d) %s() failed timer_settime()\n",
			port, __FUNCTION__);		
		return;
	}

	/*if (debug)
		ast_log(LOG_DEBUG, "port=(%d) %s() started timer_id(%d) to expire in %d seconds.\n",
			port, __FUNCTION__, (int)wdog_timer[port], GSM_WAIT_TIMEOUT);
	*/
}

void gsm_stop_check_timer(int port)
{
	struct itimerspec ts;
	int status;

	if (!gsm_port_is_valid(port)) {
		ast_log(LOG_ERROR, "port=(%d): %s() invalid port number\n",
			port, __FUNCTION__);
		return;
	}

	ts.it_value.tv_sec = 0;
	ts.it_value.tv_nsec = 0;
	ts.it_interval.tv_sec = 0;
	ts.it_interval.tv_nsec = 0;

	if (wdog_timer[port] == INVALID_TIMER_ID) {
		ast_log(LOG_WARNING, "port=(%d) %s() already destroyed\n",
			port, __FUNCTION__);
		return;
	}

	if ((status = timer_settime(wdog_timer[port], 0, &ts, 0)) < 0) {
		ast_log(LOG_ERROR, "port=(%d) %s() failed timer_settime()\n",
			port, __FUNCTION__);		
		return;
	}

	if (debug)
		ast_log(LOG_DEBUG, "port=(%d) %s() stopped timer_id(%d)\n",
			port, __FUNCTION__, (int)wdog_timer[port]);
}

void gsm_destroy_check_timer(int port)
{
	if (!gsm_port_is_valid(port)) {
		ast_log(LOG_ERROR, "port=(%d): %s() invalid port number\n",
			port, __FUNCTION__);
		return;
	}

	if (wdog_timer[port] == INVALID_TIMER_ID) {
		if(debug) ast_log(LOG_WARNING, "port=(%d) %s() already destroyed\n",
			port, __FUNCTION__);
		return;
	}

	gsm_stop_check_timer(port);

	if (debug)
		ast_log(LOG_DEBUG, "port=(%d) %s() destroyed timer_id(%d)\n",
			port, __FUNCTION__, (int)wdog_timer[port]);

	timer_delete(wdog_timer[port]);
	wdog_timer[port] = INVALID_TIMER_ID;
}

void gsm_masq_priv(void * oldpriv,void * newpriv)
{
	struct ctlport * port = getctlport_bypriv(oldpriv);

	if (port) {
		port->priv = newpriv;
	}
}

#if 0
int wait4mod(struct ctlport *ctl, fd_set *wfds, fd_set *rfds, int secs)
{
	int fd;
	struct timeval tv;
	int ret;

	if (!(ctl->fpw) || !(ctl->fpr_cmd)) return -1;
	fd=fileno( wfds?ctl->fpw:ctl->fpr_cmd);
	
	tv.tv_sec= secs ; /*wait X seconds*/
	tv.tv_usec=0;

	if (fd<0) return -1;

	FD_ZERO(wfds?wfds:rfds);
	FD_SET(fd, wfds?wfds:rfds);

	ret=select(fd+1, rfds, wfds, NULL, &tv);

	if (ret==0) {
		if (debug) ast_log(LOG_DEBUG, "port=(%d): select timed out in wait4mod\n", ctl->port);
		return -1;
	}

	if (ret<0) {
		ast_log(LOG_ERROR, "port=(%d): select returned error in wait4mod, error(%s)\n", ctl->port, strerror(errno));
		return -2;
	}

	return 1;
}

int wait4mod_r(struct ctlport *ctl, int secs)
{
	fd_set fds;
	return wait4mod(ctl, NULL, &fds, secs);
}

/* As we are using the PLX directly we don't need to wait for write interrupt !!! --- ALLO_GSM -- 23-09-2011 */
int wait4mod_w(struct ctlport *ctl, int secs)
{
	fd_set fds;
	return wait4mod(ctl, &fds, NULL, secs);
}
#endif

int cem_plx_cmd_read ( int port , char *sting){

	
}



int cem_plx_audio_file_write ( struct ctlport *ctl )
{

	PLXCM_COMMAND     *pCmd=NULL;
	PLX_DEVICE_OBJECT *pDevice = &Gbl_DeviceObj[((ctl->port-1)/4)];//PAWAN
	char buffer[256]="";
		/* Here we will read the sring of data from the file, and we write it to the PLX */
		FILE *fp = fopen("/home/projects/gsm/morning_coffee.alaw" , "r");
		if(!fp){
			fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] Failed to open the audio file..... exiting the function !!!!! \n" , __FILE__ , __func__ , __LINE__  );
			exit(-1);
		} else {
			/* We can able to open the file........*/
			char string[256]="";
			while(!feof(fp)){
				memset(&string , 0x00 , sizeof(string) );
				int i ; 
				int ret = fread(string , sizeof(string) -1 , 1 , fp );
				//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] End of for loop ret '%d' sizeof(string) '%d' string '%s' \n" , __FILE__ , __func__ , __LINE__ , ret , sizeof(string) , string );
				for(i = 0 ; i < 255 ; i++){
					memset(&buffer , 0x00 , sizeof(buffer) );
					snprintf(buffer , sizeof(buffer) -1 , "eb %s %02x" , ctl->audio_add_write , string[i] );
					time_t result;
					struct tm tm;
					char timenow[256]="";
					result = time(NULL);
					localtime_r(&result , &tm );
					strftime (timenow ,sizeof (timenow),"%I:%M:%S %p %Z",&tm);
					//fprintf(stderr,"'%s':'%s':'%s':'%d'[ALLO_GSM] buffer '%s'  and hex '%02x' i '%d' \n" , timenow , __FILE__ , __func__ , __LINE__ , buffer , string[i] , i );
					pCmd = CmdLine_CmdAdd( buffer );
        				Cmd_MemWrite( pDevice, pCmd );
#ifndef COMMAND_DELETE_END /* ALLO_GSM Deleting ALL commands at the end */
					if(pCmd){
						CmdLine_ArgDeleteAll(pCmd);
						free(pCmd);
						pCmd = NULL;
					}
#endif
					//usleep(5);
				}
				//nanosleep(NULL , NULL);
				usleep(9999);
				usleep(9999);
				usleep(999);
				usleep(99);
			}
			fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] We have completed the file playing !!!!\n" , __FILE__ , __func__ , __LINE__  );
		}
}

int audio_fifo_write = -1;
int audio_fifo_read = -1;

int cem_plx_audio_write ( int port , char *s , int len )
{

	int i = 0;
	char buffer[128]="";
	char audio_buff[len+1];/* This is to read from the PLX same time as we write to the PLX --- ALLO_GSM -- 26-12-2011 */
	struct ctlport *ctl=getctlport(port);
	PLXCM_COMMAND     *pCmd=NULL , *pCmd_read = NULL ;
	PLX_DEVICE_OBJECT *pDevice = &Gbl_DeviceObj[((port-1)/4)];//PAWAN

#if 1 /* This is the original code...... */
	memset( buffer , 0x00 , sizeof(buffer) );
	memset( audio_buff , 0x00 , sizeof(audio_buff) );
	/* This for loop is for the test ..... Latter this has to be moved to the Ping-Pong Buffer -- ALLO_GSM -- 30-09-2011 */
	//char str[8];
	//snprintf(str , 8 , "%d" , len );
	if(debug > 10)
		ast_log(LOG_ERROR , "[ALLO_GSM] writing data of '%d' bytes \n" , len );
	//if (send(ctl->soc_client_fd, str, strlen(str), 0) == -1) {
	//	perror(" ( Cem_IntRead ) send");
	//}
	ctl->write_pkt_len = len;
	char buffer_read[128]="";
	memset( buffer_read , 0x00 , sizeof(buffer_read) );
//	snprintf(buffer_read , sizeof(buffer_read) -1 , "db %s %02x" , ctlport_array[port].audio_add_read );/* This is to read from the PLX same time as we write to the PLX --- ALLO_GSM -- 26-12-2011 */
	snprintf(buffer_read , sizeof(buffer_read) -1 , "db %s" , ctlport_array[port].audio_add_read );/* This is to read from the PLX same time as we write to the PLX --- ALLO_GSM -- 26-12-2011 */
	pCmd_read = CmdLine_CmdAdd( buffer_read );/* This is to read from the PLX same time as we write to the PLX --- ALLO_GSM -- 26-12-2011 */

	for (i = 0 ; i < len ; i++){
		memset(&buffer , 0x00 , sizeof(buffer) );
		snprintf(buffer , sizeof(buffer) -1 , "eb %s %02x" , ctl->audio_add_write , s[i] );
		if(debug == 25)
			fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] we are witing audio '%s' \n" , __FILE__ , __func__ , __LINE__ , buffer );
		pCmd = CmdLine_CmdAdd( buffer );
		Cmd_MemWrite( pDevice, pCmd );
		audio_buff[i] = gsm_read_audio(pCmd_read);/* This is to read from the PLX same time as we write to the PLX --- ALLO_GSM -- 26-12-2011 */

#ifndef COMMAND_DELETE_END /* ALLO_GSM Deleting ALL commands at the end */
		if(pCmd){
			CmdLine_ArgDeleteAll(pCmd);
			free(pCmd);
			pCmd = NULL;
		}
#endif

	}
#endif
	//fprintf(stderr,"[ALLO_GSM] we are deleting pCmd '%p' (1)\n" , pCmd );


#ifndef COMMAND_DELETE_END /* ALLO_GSM Deleting ALL commands at the end */
	if(pCmd_read){
		CmdLine_ArgDeleteAll(pCmd_read);
		free(pCmd_read);
		pCmd_read = NULL;
	}
#endif

#if 1 /* This is to read from the PLX same time as we write to the PLX --- ALLO_GSM -- 26-12-2011 */
	/* After reading from the PLX write the audio data to the FIFO ..... */
	if (access(ctl->soc_filename, F_OK) == -1) {
		int res = mkfifo( ctl->soc_filename , 0777);
		if (res != 0) {          
			fprintf(stderr, "Could not create fifo %s\n" , ctl->soc_filename );
		//exit(1);                          
		}             
	}
	int audio_fifo_fd = open( ctl->soc_filename , O_RDWR );
	if(audio_fifo_fd < 0)
		perror("open audio fifo ");
	if (write( audio_fifo_fd , audio_buff , len ) <= 0 )/* This is for sending entire audio data to FIFO */
	{
	      if(debug){
        	fprintf(stderr,"[ALLO_GSM] Did we failed for writing ?????? \n");
		perror("Failed in writing audio ");
               }
	}
	close( audio_fifo_fd );/* As we send successfully close the write FIFO ..... */
#endif
	return i;
}

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

int cem_plx_audio_read ( int port , char *s , int len )
{

	int i = 0;
	char c;
	char buffer[128]="";
	struct ctlport *ctl=getctlport(port);
	PLXCM_COMMAND     *pCmd=NULL;
	struct sockaddr_in client_addr;

	memset(&buffer , 0x00 , sizeof(buffer) );

#if 1
	/* This block is for reading the FIFO */
		//ast_log(LOG_ERROR , "[ALLO_GSM] Are we looping in while loop ?????????????????????????????????????????????????????????????????????????????? \n" );

		//ast_log(LOG_ERROR , "[ALLO_GSM] Do we need to read the PLX for audio ????????...... \n" );
		len = ctl->write_pkt_len;
		int addr_len = sizeof(struct sockaddr);
		char str[32]="";
		char rcvbuf[162]="";
		memset(rcvbuf , 0x00 , 162 );
		//int n = read(ctl->audio_fd, str, 32);/* this is for test data to read from FIFO */
		int n = read(ctl->audio_fd, s, len); /* This is for reading the entire 160 bytes of audio data from FIFO */
		//int n = recv(ctl->audio_fd, rcvbuf, len , 0 );/* This is for reading UNIX socket for the audio data... */
		//int n = recvfrom(ctl->audio_fd , (void *)&rcvbuf , len , 0 , (struct sockaddr *)&client_addr , &addr_len );/* This is for reading the UDP soclet */
		if (n < 0) {
			perror("( cem_plx_audio_read: ) recv");
		/*} else {
			memcpy( s , rcvbuf , n );
		*/}
		//len = atoi(rcvbuf);
		//fprintf(stderr,"[ALLO_GSM] atoi(rcvbuf) = '%d' \n" , len );
		return len;
#endif

#if 0
	/* This blok is for reading the rtp from PLX device..... */
	snprintf(buffer , sizeof(buffer) -1 , "db %s %02x" , ctl->audio_add_read );
	//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] we are reading with command '%s'\n" , __FILE__ , __func__ , __LINE__ , buffer );
	pCmd = CmdLine_CmdAdd( buffer );

	for (i = 0 ; i < len ; i++){
		c = gsm_read_audio(pCmd);
		if(debug == 25)
			fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] we got from GSM '%x' no. of byte '%d' \n" , __FILE__ , __func__ , __LINE__ , c , i );
		s[i] = c;
	}
#endif
	if(debug > 10)
		ast_log(LOG_ERROR , "[ALLO_GSM] reading data of '%d' bytes \n" , len );
	//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] we are returning i '%d'\n" , __FILE__ , __func__ , __LINE__ , i );
	return i;
	//return 160;
}


int cem_plx_sms_write ( int port , char *s , int len)
{

	int i = 0;
	char buffer[128]="";
	char sms_buff[len+1];
	struct ctlport *ctl=getctlport(port);
	PLXCM_COMMAND     *pCmd=NULL; 
	PLX_DEVICE_OBJECT *pDevice = &Gbl_DeviceObj[((port-1)/4)];//PAWAN

	if (!ctl) return -1;
	if ((at_debug[port]>0)||debug) {
		ast_verbose(CHAN_GSM_VERBOSE_PREFIX "\tP(%d)>>> SMS WR (%s) (LEN: %d) \n", port, s, len);
	}

	if(debug) ast_verbose("[ALLO_GSM] Remove this line... %s\n", s);

	memset( buffer , 0x00 , sizeof(buffer) );
	memset( sms_buff , 0x00 , sizeof(sms_buff) );

	if(debug > 10)
		ast_log(LOG_ERROR , "[ALLO_GSM] writing data of '%d' bytes \n" , len );

	for (i = 0 ; i < len ; i++){
		memset(&buffer , 0x00 , sizeof(buffer) );
		snprintf(buffer , sizeof(buffer) -1 , "eb %s %02x" , ctl->cmd_add, s[i] );
		if(debug) ast_verbose("[ALLO_GSM] remove this also %s\n", buffer);
		pCmd = CmdLine_CmdAdd( buffer );
		Cmd_MemWrite( pDevice, pCmd );
#ifndef COMMAND_DELETE_END /* ALLO_GSM Deleting ALL commands at the end */
		if(pCmd){
			CmdLine_ArgDeleteAll(pCmd);
			free(pCmd);
			pCmd = NULL;
		}
#endif
		usleep(1);
	}
return 1;
}


int cem_plx_cmd_write ( int port , char *string){
	int count = 0;
	char send[128];
	PLXCM_COMMAND     *pCmd=NULL;
	struct ctlport *ctl=getctlport(port);
	PLX_DEVICE_OBJECT *pDevice = &Gbl_DeviceObj[((port-1)/4)];//PAWAN
	char buffer[128];
	int cmd_len;

	if (!ctl) return -1;
	if ((at_debug[port]>0)||debug) {
		ast_verbose(CHAN_GSM_VERBOSE_PREFIX "\tP(%d)>>> AT CMD (%s) \n", port, string);
	}
	char *p=strchr(string,'\r');
	//if (p) *p = '\0';
//	int cmd_len = strlen(string);

	memset(buffer , 0x00 , sizeof(buffer) );
	memset(send , 0x00 , sizeof(send) );

	if (p=strchr(string,'\n'))
		snprintf(buffer , sizeof(buffer) - 1 , "%s" , string );
	else
		snprintf(buffer , sizeof(buffer) - 1 , "%s\n" , string );


	cmd_len = strlen(buffer);

	for(count = 0; count < cmd_len ; count++){
		memset(send , 0x00 , sizeof(send) );
		snprintf( send , sizeof(send) - 1 , "eb %s %02x" , ctl->cmd_add , buffer[count] );

		if(debug) ast_verbose("[ALLO_GSM] remove this also %s\n", send);
		if(debug > 10)
			ast_log(LOG_ERROR , "[ALLO_GSM] string '%s' , ctl->cmd_add '%s' send '%s' \n" , string , ctl->cmd_add , send );
		pCmd = CmdLine_CmdAdd( send );
		//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] before sending pCmd->NumArgs '%d' \n" , __FILE__ , __func__ , __LINE__ , pCmd->NumArgs );
		Cmd_MemWrite( pDevice, pCmd );
		usleep(1);
#ifndef COMMAND_DELETE_END /* ALLO_GSM Deleting ALL commands at the end */
		if(pCmd){
			CmdLine_ArgDeleteAll(pCmd);
			free(pCmd);
			pCmd = NULL;
		}
#endif
	}

	return 1;
}

int gsm_send_raw(int port, char *s, int delay)
{
	struct ctlport *ctl=getctlport(port);
	struct timeval tv;

	char send[256];

	if (!ctl) return -1;

	/*if (wait4mod_w(ctl, DEFAULT_WAIT4MOD) <0) {
		gsm_close_port(ctl->port);
		return -1;
	}*/

	strcpy(send,s);

	char *p=strchr(send,'\n');
	if (p) *p='\0';

	if ((debug) && (strcmp("\r", send))) { 
        	gettimeofday(&tv, NULL);
		ast_log(LOG_DEBUG, "port=(%d): seconds (%ld) microseconds (%ld) sending (%s)\n",ctl->port, (long)tv.tv_sec, (long)tv.tv_usec, send);
	}

	//ast_log(LOG_ERROR , "[ALLO_GSM] we are sending teh command '%s' and send '%s' \n" , s , send );
	cem_plx_cmd_write(port , send);

	/*if (ctl->fpw) {
		fputs(send, ctl->fpw);
		fflush(ctl->fpw);
	} else {
		ast_log(LOG_ERROR, "port=(%d): invalid write fp\n", port);
	}*/

	usleep(1000*delay);
	return 0;
}


int gsm_send_sms_end(int port)
{

	char send[256]="";
	char end=26;
	//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] are we here ???????????????\n" , __FILE__ , __func__ , __LINE__  );
	PLXCM_COMMAND     *pCmd=NULL;
	struct ctlport *ctl=getctlport(port);
	PLX_DEVICE_OBJECT *pDevice = &Gbl_DeviceObj[((port-1)/4)];//PAWAN
	if (!ctl){
		fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] ops !!!!! ctl is not available for port '%d' !!!!! \n" , __FILE__ , __func__ , __LINE__ , port );
		return -1;
	}
	//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] addess '%s' we are printing the command !!! \n" , __FILE__ , __func__ , __LINE__  , ctl->cmd_add );
/*************  Prakash
	snprintf( &send , sizeof(send) - 1 , "eb %s %02x" , ctl->cmd_add , end );
*******************/
	snprintf( (char *)&send , sizeof(send) - 1 , "eb %s %02x" , ctl->cmd_add , end );
	//cem_plx_cmd_write(port , s); 

	//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] crash %s!!!!!!!!\n" , __FILE__ , __func__ , __LINE__ , send );
	pCmd = CmdLine_CmdAdd( send );
	//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] crash (2) !!!!!!!!\n" , __FILE__ , __func__ , __LINE__  );
	Cmd_MemWrite( pDevice, pCmd );
#ifndef COMMAND_DELETE_END /* ALLO_GSM Deleting ALL commands at the end */
	if(pCmd){
		CmdLine_ArgDeleteAll(pCmd);
		free(pCmd);
		pCmd = NULL;
	}
#endif

	return 0;
}

/* put the message into the message queue to be sent by the writer thread at the right time */
int gsm_send(int port, char *s, int delay)
{
	struct gsm_cmd gsm_message;
#if 0
	int msglen = sizeof(struct gsm_cmd) - sizeof(long);
#endif
	int msglen = sizeof(struct gsm_cmd);
	struct ctlport *p=getctlport(port);

	if(!p){
		//ast_log(LOG_ERROR , "[ALLO_GSM] Ops!!! ctlport is not available !!!!! \n" );
		return -1;
	}
        if(debug)       
		ast_log(LOG_DEBUG, "gsm command (%s) \n", s);
	/* This is to make sure that all the commamds we are writing at same time when we execuate.... ALLO_GSM -- 26-12-2011 */
	cem_plx_cmd_write(port , s); 
return 0;
#if 0 //ALLO_GSM Dont know why this code was here after return */
	char *ptr=strchr(s,'\n');
	if (ptr) *ptr='\0';

	/* put the message into the outgoing message queue */
	memset(&gsm_message, 0, sizeof(gsm_message));
	gsm_message.mtype = 1;
	strcpy(gsm_message.cmd_msg, s);

#if 0
	if (msgsnd(p->write_mq_id, (void*)&gsm_message, msglen, IPC_NOWAIT) < 0) {
		ast_log(LOG_ERROR, "%d port=(%d) error queuing gsm command (%s) err(%s)\n",
			p->write_mq_id, port, gsm_message.cmd_msg, strerror(errno));
		perror("msq send :");
		return -1;
	}
#endif
	ast_verbose("WRITING ON SOCKET %d, %d\n", sizeof(msglen), msglen);
	if (write(p->write_mq_id, (void*)&gsm_message, sizeof(msglen)) < 0) {
		ast_log(LOG_ERROR, "%d port=(%d) error queuing gsm command (%s) err(%s)\n",
			p->write_mq_id, port, gsm_message.cmd_msg, strerror(errno));
		perror("msq send :");
		return -1;

	} else {
		p->modem_state = MODEM_WAIT_FOR_RESPONSE;
		if (debug)
			ast_log(LOG_DEBUG, "port=(%d) queued command (%s) on port (%d)\n",
				port, gsm_message.cmd_msg, port);
	}

	/* post the semaphore to wake up the write task */
	//sem_post(&write_sem[port]);

	return 0;
#endif
}

static int _gsm_send_command(int port, char *s, int delay)
{
	struct ctlport *p=getctlport(port);
	char msg[GSM_CMD_LENGTH+3];
	
	strncpy(p->last_cmd_sent.cmd_msg, s, GSM_CMD_LENGTH);

	memset(msg, 0, sizeof(msg));
	sprintf(msg, "%s\r\n", s);

#if defined(DEFAULT_GSM_SEND_DELAY)
	delay = DEFAULT_GSM_SEND_DELAY;
#endif
	gsm_send_raw(port,msg,delay);

	return 0;
}

int send_expect(struct ctlport *ctl, char *s, char *exp, int wait, char **resp, int delay, int waitsecs)
{
	char line[256];
	char *p=NULL;
	char send[128];
	char *e=NULL;
	int ret=0;

	if(!ctl)
		return 0;

	PLX_DEVICE_OBJECT *pDevice = &Gbl_DeviceObj[((ctl->port - 1)/4)];
//ast_verbose("************************************** URGENT:HOW: gbl_dev offset: %d\n ", (ctl->port-1)/4);//PAWAN
	if (delay<=100) delay=100;

	//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] We are reading port '%d' cmd add '%s' \n" , __FILE__ , __func__ , __LINE__ , ctl->port , ctl->cmd_add );
	if (strlen(s)) {

		memset(send, 0 , 128);
		strcpy(send,s);

		if (debug) ast_log(LOG_DEBUG, "port=(%d): send_expect (%s) (%s) (%s)\n",ctl->port, s, send, exp);

		/*if (wait4mod_w(ctl, waitsecs) <0) {
			if (debug) ast_log(LOG_DEBUG, "port=(%d): not writeable\n",ctl->port);
			gsm_close_port(ctl->port);
			return -1;
		}*/

		cem_plx_cmd_write(ctl->port , send);

		/*fputs(send, ctl->fpw);
		fputs("\r\n", ctl->fpw);
		fflush(ctl->fpw);*/

	}

	/* Instead of wait4mod to read we are calling the Intrupt Read in following do-while loop ---- ALLO_GSM -- 30-09-2011 */
	/*if ((ret=wait4mod_r(ctl, waitsecs)) <0) {
		if (debug) ast_log(LOG_DEBUG, "port=(%d): not readable\n",ctl->port);
		if (ret == -2) gsm_close_port(ctl->port);
		return -1;
	}*/
	do {

		usleep(1000*delay);
		memset(line , 0 , sizeof(line));
		//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] We are waiting to read from interupt  with line '%p' !!!!!\n" , __FILE__ , __func__ , __LINE__ , &line );

		PLXCM_COMMAND     *pCmd=NULL;
		char read[128];
		memset(read , 0x00 , sizeof(read) );
		PLX_STATUS readstat;
/***************Prakash 
		snprintf(&read , sizeof(read)-1 , "db %s" , ctl->cmd_add);
******************/
		snprintf(read , sizeof(read)-1 , "db %s" , ctl->cmd_add);
		pCmd = CmdLine_CmdAdd( read );


		//ast_log(LOG_ERROR , "[ALLO_GSM] WE are reading the address '%s'  \n" , read);

		int len=0;
		if(pCmd){
			//char *bufptr = malloc(sizeof(line));
			char *bufptr = &line[0]; 
			len=sizeof(line);
			readstat = Cem_MemRead(pDevice , pCmd , &bufptr, &len );
			
#if 0
			if((len>0) && (len < sizeof(line)))
				memcpy(line, bufptr, len);
			free(bufptr);
			bufptr=NULL;
#endif
			if(len <= 1){
				if(len==1)
					printf("READ ONLY ONE BYTE SOMETHING WRONG HERE :%x: len:%d\n", line[0], len) ;
				memset(line, 0, sizeof(line));
				len=0;
			}

			if(len > sizeof(line)){
				printf("READ MORE THAN BUFF LEN:%x: len:%d\n", line[0], len) ;
				line[sizeof(line)-1]='\0';
			}
		}

#ifndef COMMAND_DELETE_END /* ALLO_GSM Deleting ALL commands at the end */
		if(pCmd){
//			ast_log(LOG_DEBUG, "HERE freeing...\n");
			CmdLine_ArgDeleteAll(pCmd);
			free(pCmd);
			pCmd = NULL;
//			ast_log(LOG_DEBUG, "HERE after freeing...\n");
		}
#endif

		//Cem_IntRead(pDevice , ctl->port , &line);
		if(readstat !=TRUE ){
			ast_log(LOG_ERROR, "port=(%d): Response read failed for command:%s\n",ctl->port, s);
			return 0;
		}

		if(len){
			int r=0;
			if(debug)	
			printf("HERE :%x:%x: len:%d\n", line[len-1], line[len-2], len);
			if(line[len-2]==0x0c)
				line[len-2]='\0';

			if(line[len-1]==0x0c)
				line[len-1]='\0';

			p=strchr(line, '\r');
			if (p) *p='\0';

			p=strchr(line, '\n');
			if (p) *p='\0';
		}
                if(debug)
			ast_verbose("[ALLO_GSM] We got from inturupt for '%s':'%s': %d:%d :%x:%x\n", s, line, strlen(line), len, '\r', '\n');

		//ast_verbose("[ALLO_GSM] P(%d) Response '%s':'%s': %d:%d\n", ctl->port, s, line, strlen(line), len);
			
		//e=fgets(line, 255, ctl->fpr);

		//if (!e) break;
		if(len && strstr(line , "AT+CPIN=?OK"))
			snprintf(line , sizeof(line) , "OK");

		//if (debug) ast_log(LOG_DEBUG, "port=(%d): getline:%s\n",ctl->port, line);

		if (len && !strcasecmp(line, send)) {
			if (debug) ast_log(LOG_DEBUG, "port=(%d): Ignoring Echo: %s\n",ctl->port, line);
		} else if (len && !strstr(line, exp)) {
			if (debug) ast_log(LOG_DEBUG, "port=(%d): send_expect got (%s)\n",ctl->port, exp);
			usleep(1000*delay);
			//ast_log(LOG_ERROR, "[ALLO_GSM] '%s':'%s':'%d'", __FILE__ , __func__ , __LINE__);
			ret = 0;
		} else if (wait) {
			if (debug) ast_log(LOG_DEBUG, "port=(%d): send_expect %s got (%s) (%s) waiting 4 (%s)\n",ctl->port, s, send, line,exp);
			if (len && strstr(line, "KILLME")) {
				if (debug) ast_log(LOG_DEBUG, "port=(%d): send_expect got (KILLME)\n", ctl->port);
					ret = 1;
			} else 
				continue;

		} else {
			if (len && resp) {
				*resp=strdup(line);
			}

			if (debug) ast_log(LOG_DEBUG, "port=(%d): expected (%s) got (%s)\n",ctl->port, exp,line);
			usleep(1000*delay);
			//ast_log(LOG_DEBUG, "[ALLO_GSM] '%s':'%s':'%d'", __FILE__ , __func__ , __LINE__);
			ret = 1;
		}
		break; /* This is only for testing !!!!! */
	}while(0);

#if 0
	if (!e) {
		if (debug) ast_log(LOG_DEBUG, "port=(%d): Connection closed\n", ctl->port);
		return -1;
	}
#endif

return ret;
}

static void gsm_set_rx_gain(int port, struct gsm_config *gsm_cfg)
{
	struct ctlport *ctl = getctlport(port);
	char clvl_command[64];
	//int fdw = (ctlport[port].fpw) ? fileno(ctlport[port].fpw) : -1;
	int rx_gain_idx;
	int ii;

	/*if (fdw < 0) {
		ast_log(LOG_ERROR, "port=(%d): invalid port number.\n", port);
		return;
	}*/

	rx_gain_idx = (gsm_cfg) ? (gsm_cfg->rx_gain) : GSM_RX_GAIN_DEFAULT ;

	if ((rx_gain_idx < GSM_RX_GAIN_MIN) || (rx_gain_idx > GSM_RX_GAIN_MAX)) {
		ast_log(LOG_ERROR, "port=(%d) rx_gain %d is out of range, falling back to default.\n", port, rx_gain_idx);
		rx_gain_idx = GSM_RX_GAIN_DEFAULT ;
	}

	/* GSM Module directly support index method, set by AT+WBHV=8,1 */
	snprintf(clvl_command, sizeof(clvl_command)-1, "AT+WDGR=1,%d", rx_gain_idx ); /* DOC: PG: 649 */
	send_expect(ctl, clvl_command, "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
}

static void gsm_set_tx_gain(int port, struct gsm_config *gsm_cfg)
{
	struct ctlport *ctl = getctlport(port);
	char cmic_command[64];
	//int fdw = (ctlport[port].fpw) ? fileno(ctlport[port].fpw) : -1;
	int tx_gain_idx;
	int ii;

	/*if (fdw < 0) {
		ast_log(LOG_ERROR, "port=(%d): invalid port number.\n", port);
		return;
	}*/

	tx_gain_idx = (gsm_cfg) ? (gsm_cfg->tx_gain) : GSM_TX_GAIN_DEFAULT ;

	if ((tx_gain_idx < GSM_TX_GAIN_MIN) || (tx_gain_idx > GSM_TX_GAIN_MAX)) {
		ast_log(LOG_ERROR, "port=(%d) tx_gain %d is out of range, falling back to default.\n", port, tx_gain_idx);
		tx_gain_idx =  GSM_TX_GAIN_DEFAULT ;
	}

	/* GSM Module directly support index method, set by AT+WBHV=8,1, NO control made on codec AON */
	snprintf(cmic_command, sizeof(cmic_command)-1, "AT+WDGT=1,%d", tx_gain_idx ); /* DOC: PG: 649 */
	send_expect(ctl, cmic_command, "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
}

static void gsm_set_dtmf_duration(int port, struct gsm_config *gsm_cfg)
{
	struct ctlport *ctl = getctlport(port);
	char vtd_command[64];
	//int fdw = (ctlport[port].fpw) ? fileno(ctlport[port].fpw) : -1;
	int ms = 1;
	int ii;

	/*if (fdw < 0) {
		ast_log(LOG_ERROR, "port=(%d): invalid port number.\n", port);
		return;
	}*/

	ms = (gsm_cfg) ? (gsm_cfg->dtmf_duration) : GSM_DTMF_DURATION_DEFAULT ;

	if ((ms < GSM_DTMF_DURATION_MIN ) || (ms > GSM_DTMF_DURATION_MAX )) {
		ast_log(LOG_ERROR, "port=(%d) dtmf duration %d is out of range, falling back to default.\n", port, ms);
		ms = GSM_DTMF_DURATION_DEFAULT ;
	}

	snprintf(vtd_command, sizeof(vtd_command)-1, "AT+VTD=%d", ms); /* DOC: PG: 673 */
	send_expect(ctl, vtd_command, "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
}

int init_mod(struct ctlport *ctl)
{
	char *resp=NULL;
	char buf[256];


	FILE *fp=NULL;

	if (ctl->cfg->initfile && strlen(ctl->cfg->initfile) ) {
		fp=fopen(ctl->cfg->initfile,"r");
		if (!fp)
			perror(ctl->cfg->initfile);
	}

	/*if (wait4mod_w(ctl, DEFAULT_WAIT4MOD)<0) {
		if (debug) 
			ast_log(LOG_DEBUG, "wait4mod_w failed\n");
	}*/

	//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] Sending expect CPIN? !!!! \n" , __FILE__ , __func__ , __LINE__  );
	/* get the "call ready" message (unsolicited) */

#if 0
	gsm_send_raw(1, "AT+WBHV=1,0\r\n", 200);	/*To change SIM stand by mode to active mode*/
#endif

	if (send_expect(ctl, "AT+CPIN?", "+CPIN: READY", 1, NULL, 0, DEFAULT_WAIT4MOD) < 0) {
		if (debug) 
			ast_log(LOG_DEBUG, "expect Call Ready failed\n");
	}

	/* check the device */
	if (send_expect(ctl, "AT", "OK", 0, NULL, 0, DEFAULT_WAIT4MOD) <0) {
		if (debug) 
			ast_log(LOG_DEBUG, "Send_Expect AT failed\n");
	}

	/* this is the case where we have a valid initialization file */
	if (fp) {
		char line[256];

		while (fgets(line,255,fp)) {
			if ( ! strncasecmp(line,"AT", 2)) {
				char cmd[256], expect[256];
				int delay, wait;
				sscanf(line,"%s\t%s\t%d\t%d",cmd,expect,&delay,&wait);

				if (debug) 
					ast_log(LOG_DEBUG, "port=(%d): Sending: cmd(%s) expect(%s) delay(%d) wait(%d)\n",ctl->port,
						cmd, expect, delay, wait);

				if (send_expect(ctl, cmd, expect, wait, &resp,delay, DEFAULT_WAIT4MOD)<=0) {
					if (debug) 
						ast_log(LOG_DEBUG, "port=(%d): cmd(%s) expect(%s) error: %s\n",ctl->port, cmd,expect, resp);
					free(resp);
				}
			} else {
				if (debug) 
					ast_log(LOG_DEBUG, "port=(%d): ignoring line: %s\n",ctl->port, line);
			}
		}

		/* authenticate us to the SIM : PIN is a parameter we need to get from configuration */
		if (strlen(ctl->cfg->pin) && strcasecmp(ctl->cfg->pin, "none") ) {
			sprintf(buf,"AT+CPIN=\"%s\"",ctl->cfg->pin);
			if (!send_expect(ctl, buf, "OK", 0, NULL, 0, DEFAULT_WAIT4MOD)) {
				if (debug) 
					ast_log(LOG_DEBUG, "port=(%d): CPIN error: %s\n", ctl->port, ((resp) ? resp : ""));
			}
		} else {
			fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] '%d' NO PIN AUTH\n" , __FILE__ , __func__ , __LINE__ , ctl->port );
		}

		/* set sms text mode (text vs. pdu) */
		if (ctl->cfg->sms_pdu_mode) {
			send_expect(ctl, "AT+CMGF=0", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
		} else {
			send_expect(ctl, "AT+CMGF=1", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
		}

#define AEC_ENABLE
#ifdef AEC_ENABLE
		send_expect(ctl, "AT+ECHO=1,4", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD); /* 1-Activate , 4-Algo Livevibes Version 1 Philips AEC */
#endif

		/* set the short message service center if it's there and not set to "none" */
		if (strlen(ctl->cfg->smsc) && (strcasecmp(ctl->cfg->smsc, "none"))) {
			sprintf(buf,"AT+CSCA=\"%s\"", ctl->cfg->smsc);
			if (!send_expect(ctl, buf, "OK", 1, NULL,0, DEFAULT_WAIT4MOD)) {
				if (debug) 
					ast_log(LOG_DEBUG, "port=(%d): SMSC error\n", ctl->port);

			}
		} else {
			if (debug) 
				ast_log(LOG_DEBUG, "port=(%d): SMSC left blank, not setting it.\n", ctl->port);
		}
	} else {
		/* this is when we have no valid configuration file */
		if (debug) 
			ast_log(LOG_DEBUG, "port=(%d): falling back to default init\n", ctl->port);

		/*Back to default settings*/

		/* Disbale STIN Responces */
		send_expect(ctl, "AT+STSF=0", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);

		/* Enable Auto RESET of Module */
		send_expect(ctl, "AT+WRST=1,\"001:00\"", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);

		/* Set DCD to be always ON */
		send_expect(ctl, "AT&C0", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);

		/* do not echo back received chars to us */
		send_expect(ctl, "ATE0", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
		
		/* Set Format of operator in Alphanumeric mode, by default its in Numeric mode*/ 
		send_expect(ctl, "AT+COPS=3,0", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);

		/* turn on unsolicited registration status messages */
		send_expect(ctl, "AT+CREG=1", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);

		/* query the registration status now */
		//send_expect(ctl, "AT+CREG?", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD); //commented coz already doing it afterwards

		/* set the sidetone gain level */
		send_expect(ctl, "AT+SIDET=0", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
	
		/* we now set the rx/tx gain with values read from configuration */	

		/* switch on SIM card detection */
		send_expect(ctl, "AT+CSDT=1", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);

#if 0 /* USE AT+CPIN? and expect '+CPIN: READY' */
		/* enable SIM insertion status reporting */
		send_expect(ctl, "AT+CSMINS=1", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
#endif

		/* enable calling line identification presentation */
		send_expect(ctl, "AT+CLIP=1", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
		
		send_expect(ctl, "AT+WIND=255", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);

		/*Set encoding for PDU msgs*/
		send_expect(ctl, "AT+CSCS=\"PCCP437\"", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);

		/* enable dtmf detection from moduled itself.. dtmf will be received by WDDI response*/
		send_expect(ctl, "AT+WDDM=1", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
		
		/* set sms text mode (text vs. pdu) */
		if (ctl->cfg->sms_pdu_mode) {
			send_expect(ctl, "AT+CMGF=0", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
		} else {
			send_expect(ctl, "AT+CMGF=1", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD);
		}

		/* set the short message service center if it's there */
		if (strlen(ctl->cfg->smsc) && (strcasecmp(ctl->cfg->smsc, "none"))) {
			sprintf(buf,"AT+CSCA=\"%s\"", ctl->cfg->smsc);
			if (!send_expect(ctl, buf, "OK", 1, NULL,0, DEFAULT_WAIT4MOD)) {
				ast_log(LOG_ERROR, "port=(%d): SMSC error: %s\n", ctl->port, resp);
			}
		} else {
			if (debug)
				ast_log(LOG_DEBUG, "port=(%d): SMSC left blank, not setting it.\n", ctl->port);
		}

		/* authenticate us to the SIM */
		if ( strcasecmp(ctl->cfg->pin, "none") ) {
			sprintf(buf,"AT+CPIN=%s",ctl->cfg->pin);
			if (!send_expect(ctl, buf, "OK", 1, NULL,0, DEFAULT_WAIT4MOD)) {
				if (debug) 
					ast_log(LOG_ERROR, "port=(%d): WHY CPIN error: %s\n", ctl->port, resp);
#if 0
	/* FIXME */
					return -EWRONGPIN;
#else
				/* May be we have already registered with pin and now just reloading the module. second attempt to register with pin will give CME_ERROR*/
#endif
			}
		} else {
			if (debug)
				ast_log(LOG_DEBUG, "%d NO PIN AUTH\n",ctl->port);
		}
	}

	/* GSM set the mode to take index value to configure the gsm module */
	send_expect(ctl, "AT+WBHV=8,1", "OK", 1, NULL, 0, DEFAULT_WAIT4MOD); /* DOC: PG: 69 */

	/* set RX gain */
	gsm_set_rx_gain(ctl->port, ctl->cfg);

	/* set TX gain */
	gsm_set_tx_gain(ctl->port, ctl->cfg);

	/* Set DTMF Duration */
	gsm_set_dtmf_duration(ctl->port, ctl->cfg);
	ast_verbose("[ALLO_GSM] Port %d INITALIZED\n", ctl->port);
#ifndef OLD_INIT
	/* Query the IMEI number */
	//gsm_send(ctl->port, "AT+GSN", 0);

	/* Query the SIM insertion status */
	//gsm_send(ctl->port, "AT+CSMINS?", 0);

	/* Query the registration status */
	gsm_send(ctl->port, "AT+CREG?", 0);
	
	/* Query the firmware rev info */
	//gsm_send(ctl->port, "AT+GMR", 0);
	if (cbEvents)
		cbEvents(ctl->port, "GSMINIT: PORT INITIALIZED");
#endif
	return 0;
}

int init_mod_port(int port)
{
	struct ctlport *ctl = getctlport(port);

	if (ctl){
		return init_mod(ctl);
	}else{
		return -ENODEV;
	}
	return -ENODEV;
}


char *errtostr[]={
		"Module initialization error 0",
		"Module initialization error",
		"WRONG PIN/ SIM Not Present"
};

#if 0
/* writer thread to write to a port */
void *write_thread(void *p)
{
	struct ctlport *ctl=(struct ctlport*)p;
	struct gsm_cmd gsm_command;
#if 0
	int msglen = sizeof(struct gsm_cmd) - sizeof(long);
#endif
	int msglen = sizeof(struct gsm_cmd);
	int retval;

	/* loop until we're killed */
	while (ctl->state != CTLPORT_STATE_KILLME) {
		
			//ast_log(LOG_DEBUG, "port=(%d): waiting on write_sem\n", ctl->port);

		/* wait on semaphore to continue */
		//sem_wait(&write_sem[ctl->port]);

			//ast_log(LOG_DEBUG, "port=(%d): acquired write_sem\n", ctl->port);

		/* check if we need to quit */
		if (ctl->state == CTLPORT_STATE_KILLME) {
				ast_log(LOG_DEBUG, "port=(%d): leaving the write_thread()\n", ctl->port);
			break;
		}

		/* get a message from the outgoing message queue */
#if 0
		if ((retval = msgrcv(ctl->write_mq_id, (void*)&gsm_command, msglen, 1, IPC_NOWAIT)) > 0) {
			/* acquire the semaphore to let you write to the port and write the message */
				//ast_log(LOG_DEBUG, "port=(%d): waiting on avail_sem\n", ctl->port);
			//sem_wait(&avail_sem[ctl->port]);

			/* check if we need to quit */
			if (ctl->state == CTLPORT_STATE_KILLME) {
					ast_log(LOG_DEBUG, "port=(%d): leaving the write_thread()\n", ctl->port);
				break;
			}
#endif
		if ((retval = read(ctl->write_mq_id, (void*)&gsm_command, msglen)) > 0) {
			/* acquire the semaphore to let you write to the port and write the message */
				//ast_log(LOG_DEBUG, "port=(%d): waiting on avail_sem\n", ctl->port);
			//sem_wait(&avail_sem[ctl->port]);

			/* check if we need to quit */
			if (ctl->state == CTLPORT_STATE_KILLME) {
					ast_log(LOG_DEBUG, "port=(%d): leaving the write_thread()\n", ctl->port);
				break;
			}


			ast_log(LOG_DEBUG, "port=(%d): acquired avail_sem SENDING message (%s)\n", ctl->port, gsm_command.cmd_msg);
			_gsm_send_command(ctl->port, gsm_command.cmd_msg, 0);
			/* increment the number of msgs sent */
			ctl->msg_count++;
			ast_log(LOG_DEBUG, "port=(%d): acquired avail_sem SENT message (%s)\n", ctl->port, gsm_command.cmd_msg);
			/* kick the watchdog timer here */
			gsm_start_check_timer(ctl->port);
			//sem_post(&avail_sem[ctl->port]);
		} else {
			/* we should not be here */
				;//ast_log(LOG_DEBUG, "port=(%d): did not get message from queue\n", ctl->port);
		}
	}

	if(debug) fprintf(stderr, "EXITING FROM HERE %s:%s:%d...port(%d)!!!\n", __FILE__, __func__, __LINE__, ctl->port);
pthread_exit((void *)ctl->port);
	return NULL;
}
#endif

/* check the response */
int gsm_check_response(const char *cmd)
{
	return (strstr(cmd, "OK") || \
		strstr(cmd, "BUSY") || \
		strstr(cmd, "NO CARRIER") || \
		strstr(cmd, "ERROR") || \
		strstr(cmd, "CMS_ERROR") || \
		strstr(cmd, "CME_ERROR"));
}

void *read_thread(void *p)
{
	struct ctlport *ctl=(struct ctlport*)p;
#if 0
	char *line = NULL;
#else
	char line[LINE_BUFFER_SIZE] ;
#endif
	int ret;
	struct timeval tv;
	PLX_DEVICE_OBJECT *pDevice = NULL;

	if(!ctl)
		return;

	pDevice = &Gbl_DeviceObj[((ctl->port-1)/4)];
        if(debug)
	ast_log(LOG_WARNING, "[ALLO_GSM] %s:%s:%d Waitin in here before init... till modem is ready to respond **FIXME**\n", __FILE__, __func__, __LINE__);
#if 0
//MODULE LOAD PROBELM
	sleep(5);	// WAIT TILL MOD GIVES RESPONSE
#endif

//	Cem_Int_Disable(&Gbl_DeviceObj[0], &Gbl_PlxInterrupt, &Gbl_NotifyObject);

#if 0
	if(plx_initialised){
		Cem_Int_Notify_Cancel(pDevice, &Gbl_PlxInterrupt, &Gbl_NotifyObject);
		plx_initialised=0;
		fprintf(stderr,"[ALLO_GSM] Inter Locking here........ \n");
		pthread_mutex_lock(&plx_lock);
	
		ret=init_mod(ctl);

		Cem_Int_Notify_Register(pDevice, &Gbl_PlxInterrupt, &Gbl_NotifyObject);
		plx_initialised=1;
		fprintf(stderr,"[ALLO_GSM] Inter UnLocking here........ \n");
		pthread_mutex_unlock(&plx_lock);
	}else
#endif
#if 0
	int retry = 5;
init_mod_retry:
#endif
	//debug=1;
	{
		pthread_mutex_lock(&init_mod_lock);
		if(debug) ast_verbose("[ALLO_GSM] %s:%s:%d p(%d)Calling init_mod...\n", __FILE__, __func__, __LINE__, ctl->port);
#if OLD_INIT
		ret=init_mod(ctl);
#else
		usleep (200*1000);
		gsm_send_sms_end(ctl->port);
		usleep (100*1000);
		gsm_send(ctl->port, "AT+CFUN=1", 200); 
		ret =0;
#endif
		if(debug) ast_verbose("[ALLO_GSM] %s:%s:%d After init_mod...\n", __FILE__, __func__, __LINE__);
		pthread_mutex_unlock(&init_mod_lock);
		if(debug) ast_verbose("[ALLO_GSM] %s:%s:%d Completed init_mod...\n", __FILE__, __func__, __LINE__);
	}


//	Cem_Int_Enable(&Gbl_DeviceObj[0], &Gbl_PlxInterrupt, &Gbl_NotifyObject);

	if (ret<0 || ! ctl->fpr_cmd ) {
		sem_post(&ready[ctl->port]);
	        if(debug) ast_log(LOG_ERROR, "port=(%d): Error on module init, stopping read thread\n",ctl->port);

		char buf[256];

	        if(debug) sprintf(buf,"GSMINIT: PORT NOT INITIALIZED: %s", errtostr[-ret]);

		if (cbEvents) {
			if (debug) 
				ast_log(LOG_DEBUG, "port=(%d): event: %s\n",ctl->port, buf);
			cbEvents(ctl->port, buf);
		} else {
		        if(debug)
				ast_log(LOG_ERROR , "[ALLO_GSM] OPsssss !!!! cbEvents are not registered !!! \n" );
		}

	        if(debug)
			ast_log(LOG_ERROR , "[ALLO_GSM] Read Thread faild ??? ret '%d' **** \n" , ret );
		//return NULL;
	        if(debug)
			ast_log(LOG_ERROR , "[ALLO_GSM] WE are sleeping....... port '%d' \n" , ctl->port);
		while(ctl->state != CTLPORT_STATE_KILLME) {
			sleep(20000);
#if 0
			if( retry && --retry)
				goto init_mod_retry;
#endif
		}
	}

#if 0
	/* allocate buffer memory */
	if (!(line=malloc(LINE_BUFFER_SIZE))) {
		ast_log(LOG_ERROR, "port=(%d) error allocating buffer memory\n", ctl->port);
		return NULL;
	}
#endif

	/* post so that the write thread can begin operation */
	sem_post(&ready[ctl->port]);
#if OLD_INIT
	/* Query the IMEI number */
	gsm_send(ctl->port, "AT+GSN", 0);

	/* Query the SIM insertion status */
	gsm_send(ctl->port, "AT+CSMINS?", 0);

	/* Query the registration status */
	gsm_send(ctl->port, "AT+CREG?", 0);

	/* Query the firmware rev info */
	gsm_send(ctl->port, "AT+GMR", 0);

	if (cbEvents)
		cbEvents(ctl->port, "GSMINIT: PORT INITIALIZED");
#endif

	if (debug) 
		ast_log(LOG_ERROR, "port=(%d): Readthread running\n",ctl->port);

	while (ctl->state != CTLPORT_STATE_KILLME) {
		int cnt = 0;
		/*if (ctl->state == CTLPORT_STATE_SMS) {
			line[0]=fgetc(ctl->fpr_cmd);
			while (line[0] == '\n' || line[0] == '\r')
				line[0]=fgetc(ctl->fpr_cmd);
			line[1]=fgetc(ctl->fpr_cmd);
			line[2]='\0';
			ast_log(LOG_ERROR , "[ALLO_GSM] We have received the line '%s' \n" , line );
			ctl->state=CTLPORT_STATE_NONE;
		} else {*/
			memset(line, 0x00, LINE_BUFFER_SIZE);
			if ( read( ctl->fpr_cmd , line, LINE_BUFFER_SIZE-1) < 0 ){
				perror("read failed ??? ");
				ast_log(LOG_ERROR, "ctl->fpr_cmd (file descriptor): %d\n", ctl->fpr_cmd);
			}
			//ast_log(LOG_ERROR , "[ALLO_GSM] We are calling the Cem_IntRead function [1] with line '%p' \n" , line );
			if ( strlen(line) <= 0 ) {
 			
				ast_log(LOG_ERROR, "port=(%d): fgets returned NULL, exiting(strlen %d)\n", ctl->port, strlen(line));
 				ctl->state = CTLPORT_STATE_KILLME;
 				usleep(100);
 				break;
	///ast_log(LOG_ERROR, "port=(%d): fgets returned NULL, exiting\n", ctl->port);
				//return NULL;
			//	continue;
			}
/******** pawan print 
//This print will print string received from plx in raw form which contains \n \r symbols
			int i;
			ast_verbose("RECEIVED:[");
			for (i=0;i<strlen(line);++i){
				if (line[i]=='\r' )
					ast_verbose("\\r");
				else if (line[i]=='\n' )
					ast_verbose("\\n");
				else 
					ast_verbose("%c",line[i]);
			}
			ast_verbose("] \n");

*********************/
#define INCLUDE_NEWLINE 1
			char *p=strchr(line, 0x04);
			if (p) *p='\0';
#if INCLUDE_NEWLINE
#else
			p=strchr(line,'\r');
			if (p) *p='\0';
#endif

			if(debug) ast_log(LOG_ERROR , "[ALLO_GSM](1) We have received the line '%s' \n" , line );
		//}

		if ((debug) && (strlen(line))) { 
        		gettimeofday(&tv,NULL);
        if(debug)
			ast_log(LOG_DEBUG, "port=(%d): seconds (%ld) microseconds (%ld) read-event: %s\n",ctl->port, (long)tv.tv_sec, (long)tv.tv_usec, line);
		}
		if ((ctl->modem_state == MODEM_WAIT_FOR_RESPONSE) && (gsm_check_response(line))) {
			if (debug) 
				ast_log(LOG_DEBUG, "port=(%d): MODEM CHECK OK\n",ctl->port);
			ctl->modem_state = MODEM_RECEIVED_RESPONSE;
		}
#if INCLUDE_NEWLINE

		int j=0;
		while (j<50)
		{
		        if (strpbrk(line,"\r\n")==line){
		                memmove(line, line+1, strlen(line));
		        }else{
		                if (strchr(line, '\r')!=NULL){
		                        char *pbuf=strdup(line);
		                        char *lineptr=strsep (&pbuf, "\r");
					char *tmpbuf=strdup(lineptr);
		                        if (debug)printf ("\t\t\t\tSep: %s\n", tmpbuf);
					if (cbEvents && strlen(tmpbuf) ){
						cbEvents(ctl->port, tmpbuf);
					}
		                        memmove(line, pbuf, strlen(line));
		                }else{
		                        if (strstr(line, ">")){
		                                if (debug) printf("\t\t\t\tSep:%s(>)\n",line);
		                        }else{
						if (strlen(line)>0)
			                                if (debug) printf("UNKNOWN SYMBOLS received: %s(len %d) first hex 0x%X\n", line, strlen(line), line[0]);
		                        }
					if (cbEvents && strlen(line) ){
						cbEvents(ctl->port, line);
					}
		                        break;
		                }
		        }/*
			printf("String:[");
			for (i=0;i<strlen(line);++i){
				if (line[i]=='\r' )
					printf("\\r");
				else if (line[i]=='\n' )
					printf("\\n");
				else
					printf("%c",line[i]);
			}
			printf("] \n");
			*/
	                ++j;
	                if (strlen(line)==0)
				break;
		}
#else
		if (cbEvents && strlen(line) ){
			cbEvents(ctl->port, line);
		}
#endif
	}

	//ast_verbose( "AM OUT OF THE READTHREAD port(%d)\n", ctl->port);

#if 0
	if  (line) 
		free(line);
#endif

	if(debug) ast_verbose( "AM OUT OF THE READTHREAD port(%d)\n", ctl->port);

	if (debug) 
		ast_log(LOG_DEBUG, "Leaving the %s() main loop\n", __FUNCTION__);

	if(debug) fprintf(stderr, "EXITING FROM HERE %s:%s:%d...port(%d)!!!\n", __FILE__, __func__, __LINE__, ctl->port);
pthread_exit((void *)ctl->port);
	return NULL;
}

char *gsm_get_imei_number(int port)
{
	struct ctlport *ctl=getctlport(port);
	return (ctl) ? (ctl->imei) : "INVALID";
}

char *gsm_get_imsi_number(int port)
{
	struct ctlport *ctl=getctlport(port);
	return (ctl) ? (ctl->imsi) : "INVALID";
}

char *gsm_get_firmware_info(int port)
{
	struct ctlport *ctl=getctlport(port);
	return (ctl) ? (ctl->fw_rev_info) : "INVALID";
}

char *gsm_get_last_command(int port)
{
	struct ctlport *ctl=getctlport(port);
	return (ctl) ? (ctl->last_cmd_sent.cmd_msg) : "INVALID";
}

void gsm_handle_unknown_event(int port, char *event)
{
	struct ctlport *ctl=getctlport(port);

	if (debug) 
		ast_log(LOG_DEBUG, "%s() : event (%s) last command (%s)\n",
			__FUNCTION__, event, ctl->last_cmd_sent.cmd_msg);

	/* decide what to do with the "event" depending on the last command sent */
	if (!strcmp("AT+GSN", ctl->last_cmd_sent.cmd_msg)) {		// IMEI query
		strncpy(ctl->imei, event, IMEI_SZ);
	} else if (!strcmp("AT+CIMI", ctl->last_cmd_sent.cmd_msg)) { 	// IMSI query
		strncpy(ctl->imsi, event, IMSI_SZ);
	} else if (!strcmp("AT+GMR", ctl->last_cmd_sent.cmd_msg)) {	// FW query
		strncpy(ctl->fw_rev_info, event, FW_REV_SZ);
	} else if (strstr(event, "*TENGMODE")) {
		ast_verbose(CHAN_GSM_VERBOSE_PREFIX "%s\n", event);	// ENG query
		if (debug) 
			ast_log(LOG_DEBUG, "Port (%d) Diag Info (%s)\n", port, event);
	} else {
		/* handle others as they arise */
	}
}

int fd_set_blocking(int fd, int blocking) 
{
	/* Save the current flags */
	int flags = fcntl(fd, F_GETFL, 0);
	if (flags == -1)
		return 0;

	if (blocking)
		flags &= ~O_NONBLOCK;
	else
		flags |= O_NONBLOCK;

	return fcntl(fd, F_SETFL, flags) != -1;
}

void audio_read__thread ( void *b)
{

#if 0 /* This is for the UDP Sockets */
	struct ctlport *ctl = (struct ctlport *)b;
	struct sockaddr_in server_addr;
	if ((ctl->audio_fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
            perror(" ( audio_read__thread ) Socket");
            exit(1);
        }

        server_addr.sin_family = AF_INET;
        server_addr.sin_port = htons(5000+ctl->port);
        server_addr.sin_addr.s_addr = INADDR_ANY;
        memset(&(server_addr.sin_zero),8 , sizeof(server_addr.sin_zero) );

	if (bind(ctl->audio_fd , (struct sockaddr *)&server_addr,sizeof(struct sockaddr)) == -1){
            perror(" ( audio_read__thread ) Bind");
            exit(1);
        }
#endif 

#if 0 /* This block is for the Unix Socket as the thread communication */
    struct ctlport *ctl = (struct ctlport *)b;
    struct sockaddr_un local, remote;
    int len , t = sizeof(remote);
    local.sun_family = AF_UNIX;  /* local is declared before socket() ^ */
    strcpy(local.sun_path, ctl->soc_filename);
    unlink(local.sun_path);
    len = strlen(local.sun_path) + sizeof(local.sun_family);

    unlink(ctl->soc_filename);
    if ((ctl->fd_w = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
        perror(" ( gsm_audio_open ) socket");
        return -1;
    }
    //fcntl(ctl->fd_w , F_SETFL , O_NONBLOCK );
    if (bind(ctl->fd_w, (struct sockaddr *)&local, len) == -1) {
        perror(" ( gsm_audio_open ) bind");
        close(ctl->fd_w);
        return -1;
    }
    if (listen(ctl->fd_w, 5) == -1) {
        perror(" ( gsm_audio_open ) listen");
        close(ctl->fd_w);
        return -1;
    }
    if ((ctl->audio_fd = accept(ctl->fd_w, (struct sockaddr *)&remote, &t)) == -1) {
        perror(" ( audio_read_thread ) accept");
        return -1;
    }
    fprintf(stderr,"[ALLO_GSM] WE have created the audio socket server  ctl->fd_w '%d' ctl->audio_fd '%d' !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n" , ctl->fd_w , ctl->audio_fd );

#endif

#if 1 /* This block is for the FIFO as thread communication....*/
	struct ctlport *ctl = (struct ctlport *)b;
	char buf[32]="";

	int res=unlink(ctl->soc_filename);
	if ((res != 0) && (errno!=2)) {
		fprintf(stderr, "Could not unlink fifo %s\n" , ctl->soc_filename);
		perror("unlink error");
		//exit(1);
        }
	if (access(ctl->soc_filename, F_OK) == -1) {
	        res = mkfifo(ctl->soc_filename, 0777);
        	if (res != 0) {
	          if(debug)  fprintf(stderr, "Could not create AUDIO FIFO %s\n" , ctl->soc_filename );
	            //exit(1);
        	}
    	}
	ctl->audio_fd=open(ctl->soc_filename, O_RDWR );
	if(ctl->audio_fd < 0){
	if(debug)	perror("AUDIO FIFO open failed ???? ");
	}
	ctl->dummy_audio_fd = open(ctl->soc_filename , O_WRONLY | O_NDELAY );
if(debug)	fprintf(stderr,"[ALLO_GSM] are we stuck here (5) .audio_fd '%d' .dummy_audio_fd '%d' ??????????????????????????\n" , ctl->audio_fd , ctl->dummy_audio_fd );

	fcntl(ctl->audio_fd, F_SETFL, O_NONBLOCK);
	/*while (1){
		//fprintf(stderr,"[ALLO_GSM] WE are waiting for the read !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! '%d' \n" , ctl->port );
		read(ctl->audio_fd , buf , sizeof(buf) );
		//fprintf(stderr,"[ALLO_GSM] buf '%s' \n" , buf );
		usleep(10);
	}*/
#endif

if(debug)	printf("EXITING FROM HERE %s:%s:%d...!!!\n", __FILE__, __func__, __LINE__);
	//pthread_exit(NULL);
}

int mem_add[33]={
	0,
	0,
	4,
	8,
	12,
	16,
	20,
	24,
	28,
	32,
	36,
	40,
	44,
	48,
	52,
	56,
	60,
	64,
	68,
	72,
	76,
	80,
	84,
	88,
	92,
	96,
	100,
	104,
	108,
	112,
	116,
	120,
	124
};

int gsm_init_port(int port, struct gsm_config *gsm_cfg)
{
	char name[64];
	int tres=0;

	sprintf(name,"/tmp/gsm_ctl_cmd_%d_%d", port, getuid());
	int res=unlink(name);
	if ((res != 0) && (errno!=2)) {
		fprintf(stderr, "Could not unlink fifo %s(forcefully deleting here)\n" , name);
		perror("unlink error");
		/* Remove this code if above error we know and can do anything about it*/
		char temp[50];
		sprintf(temp, "rm -rf %s", name);
                //system(temp);
		/***********************************************************************/
		//exit(1);
        }

	debugfp=stderr;
	memset(&ctlport_array[port], 0x00, sizeof(struct ctlport));

	ctlport_array[port].port=port;

#if 1
// Added this to have control on init module
	if(init_mod_lock_ptr==NULL){
		pthread_mutex_init(&init_mod_lock, NULL);
		init_mod_lock_ptr = &init_mod_lock;
	}
#endif
	//ctlport[port].fpw=fopen(name, "w");
	if (access(name, F_OK) == -1) {
	        int res = mkfifo(name, 0777);
        	if (res != 0) {
	            fprintf(stderr, "Could not create fifo %s\n" , name);
	            //exit(1);
        	}
    	}
       if(debug)	
	fprintf(stderr,"[ALLO_GSM] are we stuck here (1) ??????????????????????????\n");
	ctlport_array[port].fpr_cmd=open(name , O_RDWR );
	if(ctlport_array[port].fpr_cmd < 0){
		perror("open failed ???? ");
	}
	ctlport_array[port].dummy_fifo_fd = open(name , O_WRONLY | O_NDELAY );
	ctlport_array[port].cfg=gsm_cfg;
	//fd_set_blocking(fileno(ctlport_array[port].fpr_cmd),1);
	//fcntl(ctlport_array[port].fpr_cmd , F_SETFL , ~O_NONBLOCK );
       if(debug)	
       fprintf(stderr,"[ALLO_GSM] Oh !! we passed this one ....... with fifo read id '%d' \n" , ctlport_array[port].fpr_cmd);

	/* Setting the command read and audio read/write address for this port -- SANROSH -- 22-09-2011 */
	//ast_log(LOG_ERROR , "[ALLO_GSM] cmd = (mem_add[port]) '%04x' , audio read = ((mem_add[port])+128) '%04x', audio write = ((mem_add[port])+128+1) '%04x' \n" , (mem_add[port]) , ((mem_add[port])+128) , ((mem_add[port])+128+1) );
	//snprintf( ctlport_array[port].soc_filename , sizeof(ctlport_array[port].soc_filename)-1 , "/tmp/gsm%d.soc" , port ); /* This is for the unix socket cummonication used for the IPC */
	snprintf( ctlport_array[port].soc_filename , sizeof(ctlport_array[port].soc_filename)-1 , "/tmp/gsm_audio_fifo_%d_%d" , port, getuid() ); /* This is for the FIFO communication used for the IPC */
	
	int card_index;
	char barVar;
	card_index = (port - 1) / 4;
	barVar = 'V' + card_index;
	
	ctlport_array[port].card_index = card_index;
	ctlport_array[port].barVar = barVar; 

	snprintf(ctlport_array[port].cmd_add , sizeof(ctlport_array[port].cmd_add)-1 , "%c2+%04x" , barVar, (mem_add[port-(card_index*4)]) );
	snprintf(ctlport_array[port].audio_add_read , sizeof(ctlport_array[port].audio_add_read)-1 , "%c2+%04x" ,barVar, ((mem_add[port-(card_index*4)])+128) );
	snprintf(ctlport_array[port].audio_add_write , sizeof(ctlport_array[port].audio_add_write)-1 , "%c2+%04x" ,barVar, ((mem_add[port-(card_index*4)])+128+1) );
	ast_verbose("INIT PLX addresses: cmd_add:%s audio_add_read:%s audio_add_write:%s port:%d card_index:%d barVar:%c\n", ctlport_array[port].cmd_add, ctlport_array[port].audio_add_read , ctlport_array[port].audio_add_write, port, ctlport_array[port].card_index,  ctlport_array[port].barVar);
       if(debug)	
	fprintf(stderr,"[ALLO_GSM] are we stuck here (2) ??????????????????????????\n");

	if(debug)	
		ast_log(LOG_ERROR , "[ALLO_GSM] We are initializating the port '%d' cmd_add '%s' audio_add_read '%s' audio_add_write '%s' \n" , port , ctlport_array[port].cmd_add , ctlport_array[port].audio_add_read , ctlport_array[port].audio_add_write );

	if(sem_init(&ready[port], 0, 0) != 0){
       if(debug)	
		ast_log(LOG_ERROR , "[ALLO_GSM] Failed to create the ready sem as error '%s' \n" , strerror(errno));
	}
		
	/*if(!ctlport[port].fpw || !ctlport[port].fpr) {
		ast_log(LOG_ERROR, "port=(%d): Could not open name:%s FP\n errror:%s \n", port, name, strerror(errno));
		return -1;
	} else {*/
		int ret = sem_init(&write_sem[port], 0, 0);
		if(ret < 0 )
		if(debug) ast_log(LOG_ERROR , "[ALLO_GSM] Failed to create the write sem as error '%s' \n" , strerror(errno));
		ret = sem_init(&avail_sem[port], 0, 1);
		if(ret < 0 )
	 	if(debug) ast_log(LOG_ERROR , "[ALLO_GSM] Failed to create the avail sem as error '%s' \n" , strerror(errno));

		wdog_timer[port] = INVALID_TIMER_ID;
		if(debug)	
			fprintf(stderr,"[ALLO_GSM] are we stuck here (3) ??????????????????????????\n");
//#define CTL_PATH_NAME					"/var/spool/asterisk/"
#define CTL_PATH_NAME					"/tmp/"
#if 0
		key_t key = ftok(CTL_PATH_NAME, GSM_CMD_QUEUE_KEY+port);
		if ((ctlport_array[port].write_mq_id = msgget(key, IPC_CREAT | 0660)) < 0) {
			ast_log(LOG_ERROR, "port=(%d) error creating message queue %s\n", port, strerror(errno));
		}
#endif
		pthread_create(&ctlport_array[port].t, NULL, read_thread, &ctlport_array[port]);
#if 0
		if((ctlport_array[port].write_mq_id = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
			perror("socket error");
		/* Not used directly writing to plx */
		pthread_create(&ctlport_array[port].wt, NULL, write_thread, &ctlport_array[port]); 
#endif

		/* reset the number of sent msgs to zero */
		ctlport_array[port].msg_count=0;
	//}
       if(debug)	
	fprintf(stderr,"[ALLO_GSM] are we stuck here (4) ??????????????????????????\n");

#if 1
	/* Following code we are opening the fifo for the Audio processing !!!! --- ALLO_GSM -- 01-12-2011 */

	//pthread_create(&ctlport_array[port].audio_pth, NULL, (void *)audio_read__thread, &ctlport_array[port] );
	audio_read__thread(&ctlport_array[port]);

#endif
	ctlport_array[port].write_pkt_len = 160;/* default we read the 20ms packet from the PLX -- ALLO_GSM -- 21-12-2011 */

#if 0 /* Following block is for the unix socket which is controlled by the IP phone packets.... */
	int i = 0;
        int t, len;
        struct sockaddr_un remote;
        
        if ((ctlport_array[port].soc_client_fd = socket(AF_UNIX, SOCK_STREAM, 0 )) == -1) {
                perror("(audio)socket");
                return FALSE;
        }
        fcntl(ctlport_array[port].soc_client_fd , F_SETFL , O_NONBLOCK );
        
        printf("(audio)Trying to connect...\n");
        
        remote.sun_family = AF_UNIX;
        strcpy(remote.sun_path, ctlport_array[port].soc_filename );
        len = strlen(remote.sun_path) + sizeof(remote.sun_family);
        while (connect( ctlport_array[port].soc_client_fd , (struct sockaddr *)&remote, len) == -1) {
                perror("(audio)connect");
                sleep(2);
                continue;
        }
        fprintf(stderr,"[ALLO_GSM] connected the socket '%d' going for next \n" , i );
#endif

	return 0;
}


void gsm_close_port(port)
{
	/*if (ctlport[port].fpw) {
		if (fclose(ctlport[port].fpw) < 0) {
			ast_log(LOG_ERROR, "port=(%d): could not close ctlport(w) error:%s\n",
				port, strerror(errno));
		} else {
			ctlport[port].fpw=NULL;
		}
	}*/

	/*if (ctlport_array[port].fpr_cmd) {
		if (fclose(ctlport_array[port].fpr_cmd) < 0) {
			ast_log(LOG_ERROR, "port=(%d): could not close ctlport(r) error:%s\n",
				port, strerror(errno));
		} else {
			ctlport_array[port].fpr_cmd=NULL;
		}
	}*/
}

int gsm_join_write_thread(int port){
	void *r;
	if (debug) ast_log(LOG_DEBUG, "port=(%d): calling join for writethread\n", port);
	int rc = pthread_join(ctlport_array[port].wt, &r);
	if (debug) ast_log(LOG_DEBUG, "port=(%d): writethread from port killed\n", port);
return 0;
}

int gsm_join_read_thread(int port){

	void *r;
	/* terminate the read thread if it's still around */
	char cmd_int_fifo[256];
	char wakeuptoexit[32] = "Wake up and exit";
	sprintf(cmd_int_fifo, "/tmp/gsm_ctl_cmd_%d_%d", port, getuid());

	if (access(cmd_int_fifo, F_OK)==-1) {
		int res = mkfifo(cmd_int_fifo, 0777);
		if (res != 0) {          
			fprintf(stderr, "Could not create fifo %s\n" , cmd_int_fifo);
		}             
	}	
	int cmd_fifo_fd = open(cmd_int_fifo, O_RDWR );
	if(cmd_fifo_fd < 0){
		perror("open");
	}else{
		write( cmd_fifo_fd , wakeuptoexit, strlen(wakeuptoexit)-1);
		close( cmd_fifo_fd );/* As we send successfully close the client socket..... */

		int rc = pthread_join(ctlport_array[port].t, &r);
		if (debug) ast_log(LOG_DEBUG, "port=(%d): readthread from port killed\n", port);
		return 0;
	}
return 1;
}

int gsm_shutdown_port(int port, int powercycle)
{
	int fd = -1;
	struct ctlport *ctl=getctlport(port);

	if (!ctl) return -1;

	/* hangup lingering calls (if any) */
	gsm_send_raw(port, "ATH\r\n", 100);

	if (ctlport_array[port].state==CTLPORT_STATE_KILLED) return 0;

	/*if (ctlport_array[port].fpr_cmd) 
		fd = fileno(ctlport_array[port].fpr_cmd);*/

	/* set the port state to termination */
	ctlport_array[port].state=CTLPORT_STATE_KILLME;
	
	/* destroy timer for this port */
	gsm_destroy_check_timer(port);

	/* post the avail/write semaphores so the write thread can quit */
	gsm_post_next_msg(port);
	//sem_post(&write_sem[port]);

#if 0 /* No Write thread writing directly on plx */
	gsm_join_write_thread(port);
#endif
	gsm_join_read_thread(port);

#if 0
	if (fd >= 0) {
		/* to guarantee that the read thread will get its event */
		ioctl(fd, GSM_KILNME, 0);

		if (debug) 
			ast_log(LOG_DEBUG, "port=(%d): calling join for readthread\n", port);
	}
#endif

	/* stop the audio */
	gsm_audio_stop(port);

	if(debug) ast_log(LOG_ERROR, "port=(%d): readthread from port killed\n", __LINE__);
	/* close the audio port */
	gsm_audio_close(port);

	if(debug) ast_log(LOG_ERROR, "port=(%d): readthread from port killed\n", __LINE__);

	/*if (ctlport[port].fpw) {
		// power cycle the radio 
		if (powercycle) {
			int fdw = (ctlport[port].fpw) ? fileno(ctlport[port].fpw) : -1;
			if (fdw >= 0) {
				if (debug) 
					ast_log(LOG_DEBUG, "port=(%d): begin powercycle\n", port);
				ioctl(fdw, GSM_POWERCYCLE_RADIO, 1);
				if (debug) 
					ast_log(LOG_DEBUG, "port=(%d): end powercycle\n", port);
			}
		}
	}*/

	/* destroy all semaphores, so we init them "fresh" next time */
	sem_destroy(&ready[port]);
	sem_destroy(&write_sem[port]);
	sem_destroy(&avail_sem[port]);

	if(debug) ast_log(LOG_ERROR, "port=(%d): readthread from port killed\n", __LINE__);
	/* destroy the message queue for writes */
//	msgctl(ctlport_array[port].write_mq_id, IPC_RMID, NULL);
#if 0
	close(ctlport_array[port].write_mq_id);
#endif
       char name[64];
       int res;
       sprintf(name,"/tmp/gsm_ctl_cmd_%d_%d", port, getuid());
/*
	if (ctlport_array[port].fpr_cmd>-1)
		close(ctlport_array[port].fpr_cmd);
	if (ctlport_array[port].fpr_cmd>-1)
	close(ctlport_array[port].dummy_fifo_fd);
*/
       res=unlink(name);
	if ((res != 0) && (errno!=2)) {
		fprintf(stderr, "Could not unlink fifo %s\n" , name);
		perror("unlink error");
		//exit(1);
	}
       sprintf(name,"/tmp/gsm_audio_fifo_%d_%d",port, getuid());
       res=unlink(name);
	if ((res != 0) && (errno!=2)) {
		fprintf(stderr, "Could not unlink fifo %s\n" , name);
		perror("unlink error");
		//exit(1);
       }

	ctlport_array[port].state=CTLPORT_STATE_KILLED;


	if(debug) ast_log(LOG_ERROR, "port=(%d): readthread from port killed\n", __LINE__);
	/* close all control ports (read,write) */
	gsm_close_port(port);

	if(debug) ast_log(LOG_ERROR, "port=(%d): readthread from port killed\n", __LINE__);
	return 0;
}

int gsm_wait_ready(int port)
{
	sem_wait(&ready[port]);
	if (debug) 
		ast_log(LOG_DEBUG, "port=(%d): Port is ready\n",port);
	return 0;
}

int gsm_wait_ready_with_timeout(int port, int seconds)
{
	struct timespec expiry_tv;
	int retval;
	
	if ((retval = clock_gettime(CLOCK_REALTIME, &expiry_tv)) == 0) {
		expiry_tv.tv_sec += seconds;
		while(((retval = sem_timedwait(&ready[port], &expiry_tv)) == -1) && (errno == EINTR)){
		if(debug)	ast_log(LOG_ERROR, "port=(%d): Port not ready. sem_timedwait error=(%s)\n", port, strerror(errno));
			continue;
		}
			
		if(retval==0){
			if (debug) ast_log(LOG_DEBUG, "port=(%d): Port is ready\n",port);
		} else  {
	        if(debug) ast_log(LOG_ERROR, "port=(%d): Port not ready. sem_timedwait error=(%s)\n", port, strerror(errno));
		}
	} else {
	if(debug) ast_log(LOG_ERROR, "port=(%d): Port not ready. clock_gettime error=(%s)\n", port, strerror(errno));
	}

	return (retval);
}

int gsm_do_ioctl(int port, int cmd, int arg)
{
	struct ctlport *ctl=getctlport(port);
	int fd;

	if (!ctl || !ctl->fpw) return 0;

	fd=fileno(ctl->fpw);

	if (fd<=0) return -1;

	return ioctl(fd, cmd, arg);
}

int gsm_init( void (*cbEvent)(int port, char *event), struct gsm_config gsm_cfg[MAX_GSM_PORTS], int sysdebug)
{
	struct ctlport *ctl;
	int i;

	cbEvents=cbEvent;

	if (sysdebug) {
		debug=1;
		openlog("chan_gsm", LOG_PID, LOG_INFO);
	}

#if 0
//MODULE LOAD PROBELM
//	sleep(10);
#endif
	//ast_log(LOG_ERROR , "[ALLO_GSM] We are initializing the gsm port !!!! \n" );
	for (i=1; i<MAX_GSM_PORTS; i++) {
		//fprintf(stderr,"'%s':'%s':'%d'[ALLO_GSM] gsm_cfg[i].port '%d' \n" , __FILE__ , __func__ , __LINE__ , gsm_cfg[i].port );
		if (gsm_cfg[i].port) {
			if(debug) ast_verbose("[ALLO_GSM] Waiting for gsm :%d\n", gsm_cfg[i].port);
			if (gsm_wait_ready_with_timeout(i, GSM_READY_TIMEOUT) < 0) {
				ast_log(LOG_ERROR, "gsm_cfg: error timed out initializing port %d\n", i);
				/*ctl = getctlport(i);
				if ((ctl) && (ctl->fpr)) {
					int fd = fileno(ctl->fpr);
					ioctl(fd, GSM_KILLME, 0);
				}*/
				gsm_close_port(i);
				gsm_cfg[i].port = 0;
			}
		}
	}

	/* query registration status for each port */
	for (i=1; i < MAX_GSM_PORTS; i++) 
		if (gsm_cfg[i].port) {
			gsm_send(i, "AT+CREG?", 0);
		}
	fflush(debugfp);

return 0;
}

int gsm_plx_shutdown_status(){
	return plx_shutdown;
}

void gsm_shutdown(int total_plx_devices)
{


	int i;
	PLX_DEVICE_OBJECT *pDevice = NULL;
        PLX_INTERRUPT     *PlxInterrupt= NULL;
        PLX_NOTIFY_OBJECT  *NotifyObject = NULL;

	for (i=1;i<MAX_GSM_PORTS;i++) {
		if (ctlport_array[i].port) {
			if (debug) 
				ast_log(LOG_DEBUG, "port=(%d): Shutting down port\n",i);
			gsm_shutdown_port(i, 0);
		}
	}

	pthread_mutex_destroy(&init_mod_lock);
	init_mod_lock_ptr = NULL;

	if(debug) fprintf(stderr, "ALL PORTS SHUT DOWN...\n");

	if (debug && debugfp && debugfp!=stdout && debugfp!=stderr) {
		fprintf(stderr, "DEBUG fp CLOSING...\n");
		fclose(debugfp);
	}



	plx_shutdown  = 1; /* Make the flag 1 so that plx interupt thread will come out of the loop --- ALLO_GSM -- 24-11-2011 */
	for (i=0; i<total_plx_devices; ++i){
		pDevice = &Gbl_DeviceObj[i];
		PlxInterrupt = &Gbl_PlxInterrupt[i]; 
		NotifyObject= &Gbl_NotifyObject[i];
		Cem_Int_Notify_Cancel(pDevice, PlxInterrupt, NotifyObject);
	}

#if 0
	char *cmd="ATH\r\n";
	cem_plx_sms_write(1, cmd, strlen(cmd));
	gsm_send_raw(1, cmd, 200);
#endif

//	gsm_send_raw(1, "AT+WBHV=1,1\r\n", 200);	/*To change SIM active mode to Stand by mode*/

	if(debug)fprintf(stderr, "JOINING pth_plx_thread...\n");

	void *ptr;
	int rt = pthread_join(pth_plx_thread , &ptr);
	if(debug)fprintf(stderr, "JOINED THREAD pth_plx_thread...:%d ret val:%d\n", (int )pth_plx_thread, rt);
}

#if 0
#ifndef TEST_GSM
void gsm_log(int loglevel, const char *format, ...)
{
        char buffer[512];
        va_list args;

        va_start(args, format);
        vsprintf(buffer, format, args);
        va_end(args);

        /* we do this as LOG_XXX are all macros and not real enums */
        switch (loglevel) {
                case GSM_DEBUG:
                        ast_log(LOG_DEBUG, "%s", buffer);
                        break;
                case GSM_ERROR:
                        ast_log(LOG_ERROR, "%s", buffer);
                        break;
                case GSM_WARNING:
                        ast_log(LOG_WARNING, "%s", buffer);
                        break;
                case GSM_NOTICE:
                        ast_log(LOG_NOTICE, "%s", buffer);
                        break;
		case GSM_EVENT:
                        ast_log(LOG_EVENT, "%s", buffer);
                        break;
                default:
                        ast_log(LOG_NOTICE, "%s", buffer);
                        break;
        }

        syslog(LOG_INFO, buffer);
}
#endif /* TEST_GSM */

#ifdef TEST_GSM
void test_event(int port, char* event )
{
	if (debug) 
		ast_log(LOG_DEBUG, "Port (%d) fired Event (%s)\n",port, event);
}

int main(int argc, char **argv)
{
	size_t n;
	char line[256];
	struct gsm_config configs[MAX_GSM_PORTS];
	int i;

	if (argc<2 || argc>3) {
		if (debug) ast_log(LOG_DEBUG, "Please provide at least a Port as argument, addionally provied an initscript.\n");
		exit(1);
	}

	printf("Starting %d....\n", argc);

	memset(configs, 0, sizeof(configs));

	configs[atoi(argv[1])].port = 1;
	if (argc > 2)
		strcpy(configs[atoi(argv[1])].initfile, argv[2]);

	gsm_init(test_event, configs, 1);

	while (fgets(line, 255, stdin)) {
		if (line) {
			char *p=strchr(line,'\n');
			if (p)
				*p = '\0';

			if (strncmp(line, "EOM", 3) == 0) {
				gsm_send(1, "\032", 1);
				if (debug)
					printf("ctrl-z submitted\n");

			} else {
				gsm_send(atoi(argv[1]), line, 0);
				if (debug)
					printf("command submitted\n");
			}
		}
	}

	if (debug) ast_log(LOG_DEBUG, "Exiting main\n");

}

#endif
#endif
void plx_int_read_thread ( void *arg )
{
	int card_idx = (int*)arg;	

	PLX_DEVICE_OBJECT 	*pDevice = NULL;
        PLX_INTERRUPT		*PlxInterrupt = NULL;
        PLX_NOTIFY_OBJECT  	*NotifyObject = NULL;

	pDevice = &Gbl_DeviceObj[card_idx];
	PlxInterrupt = &Gbl_PlxInterrupt[card_idx];
	NotifyObject= &Gbl_NotifyObject[card_idx];

	memset(&Gbl_PlxInterrupt[card_idx], 0 , sizeof(PLX_INTERRUPT));
        if(debug)
		fprintf(stderr,"[ALLO_GSM] starting client thread\n");
//	start_client_soc(); //FIXME: Why its here... anywhere used??
	if(Cem_Int_Notify_Register(pDevice, PlxInterrupt, NotifyObject)==FALSE)
		plx_shutdown = 1;

	plx_initialised=1;
        if(debug)
	fprintf(stderr,"[ALLO_GSM] going for the while looooooop\n");
	while(plx_shutdown == 0){
		PLX_STATUS status = Cem_IntRead( pDevice, PlxInterrupt, NotifyObject, card_idx );
		if(status==ApiWaitTimeout){

       			if(debug)
				ast_log(LOG_ERROR, "[ALLO_GSM] GSM Int timedout reregister notification...!!!\n");
			Cem_Int_Notify_Cancel(pDevice, PlxInterrupt, NotifyObject);	
			if(Cem_Int_Notify_Register(pDevice, PlxInterrupt, NotifyObject)==FALSE){
				plx_shutdown = 1;
				break;
			}
		}
		if(status==ApiInvalidDeviceInfo){
			plx_shutdown = 1;
                	if(debug){
				ast_log(LOG_ERROR, "[ALLO_GSM] GSM interrupt failed problem reading device...!!!\n");
				fprintf(stderr,"[ALLO_GSM] GSM interrupt read failed... problem reading device...!!!\n");
                        }
			break;
		}
	}

	Cem_Int_Notify_Cancel(pDevice, PlxInterrupt, NotifyObject);	
	if(debug) fprintf(stderr, "EXITING FROM HERE %s:%s:%d...!!!\n", __FILE__, __func__, __LINE__);
pthread_exit((void *)plx_shutdown);
}
/* This function will start the thread which will constantly run the Interupt commands for the PLX -- ALLO_GSM - 21-11-2011 */
void gsm_start_int_thread ( int total_plx_devices )
{
	int i;
	for (i=0;i<total_plx_devices;++i)
		pthread_create(&pth_plx_thread , NULL , (void *)plx_int_read_thread , (void *)i );
}
