#ifndef _GSMCTL_
#define _GSMCTL_

#include <stdio.h>
#include <string.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <semaphore.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <signal.h>
//#include <syslog.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <time.h>



#include "CmdLine.h"
#include "Monitor.h"
#include "MonCmds.h"
#include "PlxApi.h"
#include "PciRegs.h"
#include "RegDefs.h"
#include "ConsFunc.h"
#include "PlxInit.h"
#include "PlxApi.h"
#include "PexApi.h"



/**********************************************
 *                 Globals
 **********************************************/
 PLXCM_COMMAND     *Gbl_pPostedCmd;
 DEVICE_NODE       *Gbl_pNodeCurrent[4];
 PLX_DEVICE_OBJECT  Gbl_DeviceObj[4];
 //PLX_INTERRUPT	    Gbl_PlxInterrupt;//pawan
 PLX_INTERRUPT	    Gbl_PlxInterrupt[4];//pawan
// PLX_NOTIFY_OBJECT  Gbl_NotifyObject;//pawan
 PLX_NOTIFY_OBJECT  Gbl_NotifyObject[4];//pawan
 PLX_DEVICE_OBJECT  CurrentGbl_DeviceObj;
 DEVICE_NODE       *CurrentGbl_pNodeCurrent[1];

/* Added for the gsm functionality defined in the GsmTestApp.c ---- SANTOSH -- 27-09-2011 */
char gsm_read_audio ( PLXCM_COMMAND *pCmd );
void gsm_write_audio ( PLXCM_COMMAND *pCmd );
VOID PciSpacesMap( PLX_DEVICE_OBJECT *pDevice , DEVICE_NODE *Gbl_pNodeCurrent, int card_idx );
VOID CommonBufferMap( PLX_DEVICE_OBJECT *pDevice );
DEVICE_NODE* DeviceSelectByIndex( U8 index , DEVICE_NODE *Gbl_pNodeCurrent );
VOID Mon_PostCommand( PLXCM_COMMAND *pCmd );
VOID PciSpacesUnmap( PLX_DEVICE_OBJECT *pDevice , DEVICE_NODE *Gbl_pNodeCurrent, int card_idx );
VOID CommonBufferUnmap( PLX_DEVICE_OBJECT *pDevice );
BOOLEAN Cem_MemRead( PLX_DEVICE_OBJECT *pDevice, PLXCM_COMMAND *pCmd , char **buffer , int *len);
U64 Cem_Single_MemRead( PLX_DEVICE_OBJECT *pDevice, PLXCM_COMMAND *pCmd );
//BOOLEAN Cem_IntRead( PLX_DEVICE_OBJECT *pDevice, int port , char *line);
PLX_STATUS Cem_IntRead( PLX_DEVICE_OBJECT *pDevice, PLX_INTERRUPT *PlxInterrupt, PLX_NOTIFY_OBJECT *NotifyObject ,int total_plx_devices);
void start_client_soc( void );

/* This function will start the thread which will constantly run the Interupt commands for the PLX -- SANTOSH - 21-11-2011 */
#define GSM_MAX_STR	255
#define MAX_GSM_PORTS   5


/* todo : move them to gsmctl.h */
#define GSM_LINE_LENGTH 	4096
#define DEFAULT_WAIT4MOD 	30
#define DEFAULT_GSM_SEND_DELAY 	0
#define GSM_CMD_QUEUE_KEY	0xCEEF0000
#define LINE_BUFFER_SIZE	4096
#define GSM_WAIT_TIMEOUT	30
#define GSM_CMD_LENGTH		255
#define INVALID_TIMER_ID	(timer_t)-1

#define GSM_TX_GAIN_MIN		0
#define GSM_TX_GAIN_MAX		255
#define GSM_TX_GAIN_DEFAULT	176

#define GSM_RX_GAIN_MIN		0
#define GSM_RX_GAIN_MAX		255
#define GSM_RX_GAIN_DEFAULT	64

#define GSM_DTMF_DURATION_MAX 6
#define GSM_DTMF_DURATION_MIN 1
#define GSM_DTMF_DURATION_DEFAULT 1


#define IMEI_SZ	32
#define IMSI_SZ 32
#define FW_REV_SZ 128	




struct gsm_config {
	int port;
	char initfile[256];
	char pin[256];
	char smsc[256];
	int sms_pdu_mode;
	int rx_gain;
	int tx_gain;
	int dtmf_duration;
	int early_media;
};

struct gsm_req {
	int type;
	unsigned param1;
	unsigned param2;
};

enum gsm_errors {
	EWRONGPIN=2
};



enum ctlport_state {
	CTLPORT_STATE_NONE=0,
	CTLPORT_STATE_SMS,
	CTLPORT_STATE_KILLME,
	CTLPORT_STATE_KILLED,
};

enum modem_state {
	MODEM_OK=0,
	MODEM_WAIT_FOR_RESPONSE,
	MODEM_RECEIVED_RESPONSE
};

/* gsm command type */
struct gsm_cmd {
	int mtype;
	char cmd_msg[GSM_CMD_LENGTH+1];
};

typedef struct gain_tbl_entry {
  int table_value;
  int codec_value;
  int radio_value;
} gain_tbl_entry_t;

const static gain_tbl_entry_t gsm_rx_gain_table[] = {
	{ .table_value =  10,  .codec_value = 0x00, .radio_value = 100 },
	{ .table_value =   9,  .codec_value = 0x00, .radio_value = 98 },
	{ .table_value =   8,  .codec_value = 0x00, .radio_value = 95 },
	{ .table_value =   7,  .codec_value = 0x00, .radio_value = 93 },
	{ .table_value =   6,  .codec_value = 0x00, .radio_value = 90 },
	{ .table_value =   5,  .codec_value = 0x01, .radio_value = 90 },
	{ .table_value =   4,  .codec_value = 0x02, .radio_value = 90 },
	{ .table_value =   3,  .codec_value = 0x03, .radio_value = 90 },
	{ .table_value =   2,  .codec_value = 0x04, .radio_value = 90 },
	{ .table_value =   1,  .codec_value = 0x05, .radio_value = 90 },
	{ .table_value =   0,  .codec_value = 0x06, .radio_value = 90 },
	{ .table_value =  -1,  .codec_value = 0x07, .radio_value = 90 },
	{ .table_value =  -2,  .codec_value = 0x08, .radio_value = 90 },
	{ .table_value =  -3,  .codec_value = 0x09, .radio_value = 90 },
	{ .table_value =  -4,  .codec_value = 0x0A, .radio_value = 90 },
	{ .table_value =  -5,  .codec_value = 0x0B, .radio_value = 90 },
	{ .table_value =  -6,  .codec_value = 0x0C, .radio_value = 90 },
	{ .table_value =  -7,  .codec_value = 0x0D, .radio_value = 90 },
	{ .table_value =  -8,  .codec_value = 0x0E, .radio_value = 90 },
	{ .table_value =  -9,  .codec_value = 0x0F, .radio_value = 90 },
	{ .table_value = -10,  .codec_value = 0x10, .radio_value = 90 },
	{ .table_value = -11,  .codec_value = 0x11, .radio_value = 90 },
	{ .table_value = -12,  .codec_value = 0x12, .radio_value = 90 },
	{ .table_value = -13,  .codec_value = 0x13, .radio_value = 90 },
	{ .table_value = -14,  .codec_value = 0x14, .radio_value = 90 },
	{ .table_value = -15,  .codec_value = 0x15, .radio_value = 90 },
	{ .table_value = -16,  .codec_value = 0x16, .radio_value = 90 },
	{ .table_value = -17,  .codec_value = 0x17, .radio_value = 90 },
	{ .table_value = -18,  .codec_value = 0x18, .radio_value = 90 },
};

const static gain_tbl_entry_t gsm_tx_gain_table[] = {
	{ .table_value =  24, .codec_value = 0x00, .radio_value = 8 },
	{ .table_value =  23, .codec_value = 0x01, .radio_value = 8 },
	{ .table_value =  22, .codec_value = 0x02, .radio_value = 8 },
	{ .table_value =  21, .codec_value = 0x03, .radio_value = 8 },
	{ .table_value =  20, .codec_value = 0x04, .radio_value = 8 },
	{ .table_value =  19, .codec_value = 0x05, .radio_value = 8 },
	{ .table_value =  18, .codec_value = 0x06, .radio_value = 8 },
	{ .table_value =  17, .codec_value = 0x07, .radio_value = 8 },
	{ .table_value =  16, .codec_value = 0x08, .radio_value = 8 },
	{ .table_value =  15, .codec_value = 0x09, .radio_value = 8 },
	{ .table_value =  14, .codec_value = 0x0A, .radio_value = 8 },
	{ .table_value =  13, .codec_value = 0x0B, .radio_value = 8 },
	{ .table_value =  12, .codec_value = 0x0C, .radio_value = 8 },
	{ .table_value =  11, .codec_value = 0x0D, .radio_value = 8 },
	{ .table_value =  10, .codec_value = 0x0E, .radio_value = 8 },
	{ .table_value =   9, .codec_value = 0x0F, .radio_value = 8 },
	{ .table_value =   8, .codec_value = 0x10, .radio_value = 8 },
	{ .table_value =   7, .codec_value = 0x11, .radio_value = 8 },
	{ .table_value =   6, .codec_value = 0x12, .radio_value = 8 },
	{ .table_value =   5, .codec_value = 0x13, .radio_value = 8 },
	{ .table_value =   4, .codec_value = 0x14, .radio_value = 8 },
	{ .table_value =   3, .codec_value = 0x15, .radio_value = 8 },
	{ .table_value =   2, .codec_value = 0x16, .radio_value = 8 },
	{ .table_value =   1, .codec_value = 0x17, .radio_value = 8 },
	{ .table_value =   0, .codec_value = 0x18, .radio_value = 8 },
	{ .table_value =  -1, .codec_value = 0x18, .radio_value = 7 },
	{ .table_value =  -2, .codec_value = 0x18, .radio_value = 7 },
	{ .table_value =  -3, .codec_value = 0x18, .radio_value = 6 },
	{ .table_value =  -4, .codec_value = 0x18, .radio_value = 5 },
	{ .table_value =  -5, .codec_value = 0x18, .radio_value = 5 },
	{ .table_value =  -6, .codec_value = 0x18, .radio_value = 4 },
	{ .table_value =  -7, .codec_value = 0x18, .radio_value = 3 },
	{ .table_value =  -8, .codec_value = 0x18, .radio_value = 3 },
	{ .table_value =  -9, .codec_value = 0x18, .radio_value = 2 },
	{ .table_value = -10, .codec_value = 0x18, .radio_value = 1 },
	{ .table_value = -11, .codec_value = 0x18, .radio_value = 1 },
	{ .table_value = -12, .codec_value = 0x18, .radio_value = 0 },
};




/* TODO : tuck all semaphores into ctlport structure */
sem_t ready[MAX_GSM_PORTS];
sem_t write_sem[MAX_GSM_PORTS];
sem_t avail_sem[MAX_GSM_PORTS];
timer_t wdog_timer[MAX_GSM_PORTS];


struct ctlport {
	FILE *fpw,*fpr;
	pthread_t t;
	pthread_t wt;
	pthread_t audio_pth;
	int fd;
	int fpr_cmd;
	int dummy_fifo_fd;
	int fd_w;
	FILE *file_w;
	int read_flag;
// SAM	char *initfile;
	int port;
	int in_use;
	struct gsm_config *cfg;
	enum ctlport_state state;
	enum modem_state modem_state;
	void *priv;
	int write_mq_id;
	int msg_count;
	struct gsm_cmd last_cmd_sent;
	char imei[IMEI_SZ+1];
// Only the SIM currently connected to the radio is queried.
	char imsi[IMSI_SZ+1]; 
	char fw_rev_info[FW_REV_SZ+1];
	
	/* Following variables are added by SANTOSH -- 22-09-2011 */
	int thread_exit; /* Flag for the client thread to exit from the loop */
	char cmd_add[32]; /* This is the address for the commands to read and write word ( 2 bytes each cycle...)*/
	char audio_add_read[32]; /* This is the address where we read the audio bytes */
	char audio_add_write[32]; /* This is the address where we write the audio bytes */
	char soc_filename[32];
	pthread_t pth; /* This is the thread id which we have used for this call..... -- SANTOSH */
	int audio_fd;
	int dummy_audio_fd;
	int soc_client_fd ; /* This is the unix client fd for the write function to make the poll --- SANTOSH -- 19-12-2011 */
	int write_pkt_len; /* This is the size of pkt we receive from the IP phone...... -- SANTOSH -- 21-12-2011 */
	/* Added for Multiple Card Support ++pawan */
	int card_index;
	char barVar;
};



struct ctlport ctlport_array[MAX_GSM_PORTS];

int gsm_send_sms_end(int port);
int gsm_send(int port, char *s, int delay);
int gsm_send_raw(int port, char *s, int delay);

int gsm_init( void (*cbEvent)(int port, char *event), struct gsm_config gsm_cfg[MAX_GSM_PORTS], int sysdebug);
void gsm_shutdown(int total_plx_devices);

int gsm_init_port(int port, struct gsm_config *gsm_cfg);
int gsm_wait_ready(int port);
int gsm_wait_ready_with_timeout(int port, int seconds);
int gsm_shutdown_port(int port, int powercycle);

void *gsm_get_priv(int port);
void gsm_put_priv(int port, void *priv);

int gsm_audio_open(int port);
void gsm_audio_close(int port);

int gsm_audio_start(int port);
int gsm_audio_stop(int port);

int gsm_set_next_simslot(int port, int slot);

int gsm_sms_enter(int port);

int gsm_port_is_valid(int port);

int gsm_port_fetch(int port);
int gsm_port_release(int port);
int gsm_port_in_use_get(int port);

int gsm_simslot_get(int port);
int gsm_simslot_next_get(int port);

int gsm_check_port(int port);
int gsm_set_led(int port, int ledstate);

int init_mod_port(int port);

int gsm_do_ioctl(int port, int cmd, int arg);
void gsm_set_debuglevel(int debuglevel);
const char *gsm_get_cme_text(int cme_code);
const char *gsm_get_cms_text(int cme_code);
void gsm_post_next_msg(int port);
void gsm_start_check_timer(int port);
void gsm_stop_check_timer(int port);
int gsm_get_channel_mask(void);
int gsm_get_channel_offset(void);
void gsm_handle_unknown_event(int port, char *event);
char *gsm_get_imei_number(int port);
char *gsm_get_imsi_number(int port);
char *gsm_get_firmware_info(int port);
int gsm_translate_channel(int *result, int channel_id);
char *gsm_get_last_command(int port);
void gsm_masq_priv(void * oldpriv,void * newpriv);
void gsm_log(int loglevel, const char *format, ...);
void gsm_close_port(int port);

struct ctlport *getctlport(int port);

void gsm_chan_read_flag( int port );
void gsm_start_int_thread ( int total_plx_devices );
int send_expect(struct ctlport *ctl, char *s, char *exp, int wait, char **resp, int delay, int waitsecs);
int cem_plx_sms_write ( int port , char *s , int len);
int gsm_plx_shutdown_status();

int cem_plx_audio_read ( int port , char *s , int len );
int cem_plx_audio_file_write ( struct ctlport *ctl );
int cem_plx_audio_write ( int port , char *s , int len );
int cem_plx_cmd_write ( int port , char *string);



#define GSM_IOC_MAGIC                      	'G'
#define GSM_SETLED                         	_IO(GSM_IOC_MAGIC, 0)
#define GSM_GETLED                         	_IO(GSM_IOC_MAGIC, 1)
#define GSM_SETSTATE                       	_IO(GSM_IOC_MAGIC, 2)
#define GSM_SETNEXTSIMSLOT                 	_IO(GSM_IOC_MAGIC, 3)
#define GSM_KILLME				_IO(GSM_IOC_MAGIC, 4)
#define GSM_PEEK_FPGA				_IO(GSM_IOC_MAGIC, 5)
#define GSM_POKE_FPGA				_IO(GSM_IOC_MAGIC, 6)
#define GSM_POWERCYCLE_RADIO			_IO(GSM_IOC_MAGIC, 7)
#define GSM_SET_RX_GAIN				_IO(GSM_IOC_MAGIC, 8)
#define GSM_SET_TX_GAIN				_IO(GSM_IOC_MAGIC, 9)


#define LED_GREEN                          (0x1 << 0)
#define LED_RED                            (0x1 << 1)
#define LED_OFF                            (0x1 << 2)
#define LED_ON                             (0x1 << 3)
#define LED_BLINK                          (0x1 << 4)

#define STATE_CONNECTED                    (0x1 << 0)
#define STATE_HANGUP                       (0x1 << 1)

#define CHAN_GSM_VERBOSE_PREFIX			VERBOSE_PREFIX_1

#define GSM_DEBUG	__LOG_DEBUG
#define GSM_ERROR	__LOG_ERROR
#define GSM_WARNING	__LOG_WARNING
#define GSM_NOTICE	__LOG_NOTICE
#define GSM_EVENT	__LOG_EVENT

#define GSM_READY_TIMEOUT			15


#endif
